@@ -28,7 +28,7 @@ Requires:  Trac 0.9b1 from http://trac.edgewall.org/
            or PostGreSQL 8.4 from http://www.postgresql.org/
            or SQLite 3 from http://www.sqlite.org/
 
-$Id: bugzilla2trac.py 12500 2014-02-12 20:54:59Z rjollos $
+$Id: bugzilla2trac.py 13166 2014-10-26 04:24:20Z rjollos $
 """
 
 from __future__ import with_statement
@@ -902,16 +902,14 @@ def convert(_db, _host, _user, _password, _env, _force):
         users = mysql_cur.fetchall()
     else:
         users = ()
-    htpasswd = file("htpasswd", 'w')
-    for user in users:
-        if user['login_name'] in LOGIN_MAP:
-            login = LOGIN_MAP[user['login_name']]
-        else:
-            login = user['login_name']
+    with open('htpasswd', 'w') as f:
+        for user in users:
+            if user['login_name'] in LOGIN_MAP:
+                login = LOGIN_MAP[user['login_name']]
+            else:
+                login = user['login_name']
+            f.write(login + ':' + user['cryptpassword'] + '\n')
 
-        htpasswd.write(login + ":" + user['cryptpassword'] + "\n")
-
-    htpasswd.close()
     print "  Bugzilla users converted to htpasswd format, see 'htpasswd'."
 
     print "\nAll tickets converted."
