@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006 Edgewall Software
+# Copyright (C) 2006-2013 Edgewall Software
 # Copyright (C) 2006 Matthew Good <matt@matt-good.net>
 # All rights reserved.
 #
@@ -17,15 +17,11 @@
 
 import errno
 import fileinput
-import sys
-from optparse import OptionParser
 from getpass import getpass
+from hashlib import md5
+from optparse import OptionParser
+import sys
 
-# The md5 module is deprecated in Python 2.5
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
 
 def ask_pass():
     pass1 = getpass('New password: ')
