@@ -19,7 +19,7 @@ from trac.core import *
 from trac.perm import IPermissionPolicy
 
 revision = "$Rev: 12500 $"
-url = "$URL: http://svn.edgewall.org/repos/trac/tags/trac-1.0.2/sample-plugins/permissions/public_wiki_policy.py $"
+url = "$URL: http://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/permissions/public_wiki_policy.py $"
 
 class PublicWikiPolicy(Component):
     """Allow public access to some wiki pages.
