@@ -19,7 +19,7 @@ from trac.ticket.api import ITicketActionController, TicketSystem
 from trac.perm import IPermissionRequestor
 
 revision = "$Rev: 12164 $"
-url = "$URL: http://svn.edgewall.org/repos/trac/tags/trac-1.0.2/sample-plugins/workflow/StatusFixer.py $"
+url = "$URL: http://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/workflow/StatusFixer.py $"
 
 class StatusFixerActionController(Component):
     """Provides the admin with a way to correct a ticket's status.
