@@ -13,8 +13,12 @@
 # history and logs, available at http://trac.edgewall.org/log/.
 
 import os
+import re
+import time
+import unittest
 
-from trac.tests.functional import *
+from trac.tests.functional import FunctionalTwillTestCaseSetup, \
+                                  internal_error, tc
 from trac.util import create_file
 
 
@@ -57,7 +61,6 @@ class RaiseExceptionPlugin(Component):
             return True
 
     def process_request(self, req):
-        print 'maybe?'
         if req.args.get('report') == 'tho':
             global url
             url = 'http://trac-hacks.org/wiki/HelloWorldMacro'
@@ -116,7 +119,7 @@ class RegressionTestRev6017(FunctionalTwillTestCaseSetup):
                     os.unlink(filename)
 
 
-class RegressionTestTicket3833a(FunctionalTestCaseSetup):
+class RegressionTestTicket3833a(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Test for regression of http://trac.edgewall.org/ticket/3833 a"""
         # Assume the logging is already set to debug.
@@ -134,7 +137,7 @@ class RegressionTestTicket3833a(FunctionalTestCaseSetup):
                             % debug1)
 
 
-class RegressionTestTicket3833b(FunctionalTestCaseSetup):
+class RegressionTestTicket3833b(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Test for regression of http://trac.edgewall.org/ticket/3833 b"""
         # Turn logging off, try to log something, and verify that it does
@@ -158,7 +161,7 @@ class RegressionTestTicket3833b(FunctionalTestCaseSetup):
                          % debug2)
 
 
-class RegressionTestTicket3833c(FunctionalTestCaseSetup):
+class RegressionTestTicket3833c(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Test for regression of http://trac.edgewall.org/ticket/3833 c"""
         # Turn logging back on, try to log something, and verify that it
@@ -285,6 +288,10 @@ class RegressionTestTicket6318(FunctionalTwillTestCaseSetup):
             self._testenv.adduser(u'joé')
             self._tester.login(u'joé')
             self._tester.go_to_front()
+            # when failed to retrieve session, FakeSession() and FakePerm()
+            # are used and the req.perm has no permissions.
+            tc.notfind(internal_error)
+            tc.notfind("You don't have the required permissions")
             self._tester.logout()
             # finally restore expected 'admin' login
             self._tester.login('admin')
@@ -331,6 +338,45 @@ class RaiseExceptionPlugin(Component):
             env.config.set('components', 'RaiseExceptionPlugin.*', 'disabled')
 
 
+class RegressionTestTicket11503a(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Test for regression of http://trac.edgewall.org/ticket/11503 a"""
+        base = self._tester.url
+
+        tc.go(base + '/notf%C5%91und/')
+        tc.notfind(internal_error)
+        tc.url(re.escape(base + '/notf%C5%91und') + r'\Z')
+
+        tc.go(base + '/notf%C5%91und/?type=def%C3%A9ct')
+        tc.notfind(internal_error)
+        tc.url(re.escape(base + '/notf%C5%91und?type=def%C3%A9ct') + r'\Z')
+
+        tc.go(base + '/notf%C5%91und/%252F/?type=%252F')
+        tc.notfind(internal_error)
+        tc.url(re.escape(base + '/notf%C5%91und/%252F?type=%252F') + r'\Z')
+
+
+class RegressionTestTicket11503b(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Test for regression of http://trac.edgewall.org/ticket/11503 b"""
+        env = self._testenv.get_trac_environment()
+        try:
+            env.config.set('mainnav', 'wiki.href',
+                           u'/wiki/SändBõx?action=history&blah=%252F')
+            env.config.save()
+            # reloads the environment
+            env = self._testenv.get_trac_environment()
+
+            self._tester.go_to_front()
+            tc.notfind(internal_error)
+            tc.find(' href="/wiki/S%C3%A4ndB%C3%B5x\?'
+                    'action=history&amp;blah=%252F"')
+        finally:
+            env.config.remove('mainnav', 'wiki.href')
+            env.config.save()
+
+
+
 def functionalSuite(suite=None):
     if not suite:
         import trac.tests.functional
@@ -347,8 +393,13 @@ def functionalSuite(suite=None):
     suite.addTest(RegressionTestTicket3663())
     suite.addTest(RegressionTestTicket6318())
     suite.addTest(RegressionTestTicket11434())
+    suite.addTest(RegressionTestTicket11503a())
+    suite.addTest(RegressionTestTicket11503b())
     return suite
 
 
+suite = functionalSuite
+
+
 if __name__ == '__main__':
-    unittest.main(defaultTest='functionalSuite')
+    unittest.main(defaultTest='suite')
