@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2008 Edgewall Software
+# Copyright (C) 2005-2013 Edgewall Software
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
 #
@@ -12,31 +12,37 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 #
-# Include a basic SMTP server, based on L. Smithson 
+# Include a basic SMTP server, based on L. Smithson
 # (lsmithson@open-networks.co.uk) extensible Python SMTP Server
 #
 # This file does not contain unit tests, but provides a set of
 # classes to run SMTP notification tests
 #
 
+import base64
+import os
+import quopri
+import re
 import socket
 import string
 import threading
-import re
-import base64
-import quopri
+import unittest
 
+from trac.config import ConfigurationError
+from trac.notification import SendmailEmailSender, SmtpEmailSender
+from trac.test import EnvironmentStub
 
 LF = '\n'
 CR = '\r'
-email_re = re.compile(r"([\w\d_\.\-])+\@(([\w\d\-])+\.)+([\w\d]{2,4})+")
+SMTP_TEST_PORT = 7000 + os.getpid() % 1000
+email_re = re.compile(r'([\w\d_\.\-])+\@(([\w\d\-])+\.)+([\w\d]{2,4})+')
 header_re = re.compile(r'^=\?(?P<charset>[\w\d\-]+)\?(?P<code>[qb])\?(?P<value>.*)\?=$')
 
 
 class SMTPServerInterface:
     """
-    A base class for the imlementation of an application specific SMTP
-    Server. Applications should subclass this and overide these
+    A base class for the implementation of an application specific SMTP
+    Server. Applications should subclass this and override these
     methods, which by default do nothing.
 
     A method is defined for each RFC821 command. For each of these
@@ -44,17 +50,17 @@ class SMTPServerInterface:
     client. The 'data' method is called after all of the client DATA
     is received.
 
-    If a method returns 'None', then a '250 OK'message is
+    If a method returns 'None', then a '250 OK' message is
     automatically sent to the client. If a subclass returns a non-null
     string then it is returned instead.
     """
 
     def helo(self, args):
         return None
-    
+
     def mail_from(self, args):
         return None
-        
+
     def rcpt_to(self, args):
         return None
 
@@ -66,11 +72,11 @@ class SMTPServerInterface:
 
     def reset(self, args):
         return None
-    
+
+
 #
 # Some helper functions for manipulating from & to addresses etc.
 #
-
 def strip_address(address):
     """
     Strip the leading & trailing <> from an address.  Handy for
@@ -80,6 +86,7 @@ def strip_address(address):
     end = string.index(address, '>')
     return address[start:end]
 
+
 def split_to(address):
     """
     Return 'address' as undressed (host, fulladdress) tuple.
@@ -88,7 +95,7 @@ def split_to(address):
     start = string.index(address, '<') + 1
     sep = string.index(address, '@') + 1
     end = string.index(address, '>')
-    return (address[sep:end], address[start:end],)
+    return address[sep:end], address[start:end]
 
 
 #
@@ -99,7 +106,7 @@ class SMTPServerEngine:
     Server engine that calls methods on the SMTPServerInterface object
     passed at construction time. It is constructed with a bound socket
     connection to a client. The 'chug' method drives the state,
-    returning when the client RFC821 transaction is complete. 
+    returning when the client RFC821 transaction is complete.
     """
 
     ST_INIT = 0
@@ -108,10 +115,10 @@ class SMTPServerEngine:
     ST_RCPT = 3
     ST_DATA = 4
     ST_QUIT = 5
-    
+
     def __init__(self, socket, impl):
-        self.impl = impl;
-        self.socket = socket;
+        self.impl = impl
+        self.socket = socket
         self.state = SMTPServerEngine.ST_INIT
 
     def chug(self):
@@ -130,16 +137,16 @@ class SMTPServerEngine:
             # this out.
             while not completeLine:
                 try:
-                    lump = self.socket.recv(1024);
+                    lump = self.socket.recv(1024)
                     if len(lump):
                         data += lump
-                        if (len(data) >= 2) and data[-2:] == '\r\n':
+                        if len(data) >= 2 and data[-2:] == '\r\n':
                             completeLine = 1
                             if self.state != SMTPServerEngine.ST_DATA:
                                 rsp, keep = self.do_command(data)
                             else:
                                 rsp = self.do_data(data)
-                                if rsp == None:
+                                if rsp is None:
                                     continue
                             self.socket.send(rsp + "\r\n")
                             if keep == 0:
@@ -150,8 +157,7 @@ class SMTPServerEngine:
                         return
                 except socket.error:
                     return
-        return
-            
+
     def do_command(self, data):
         """Process a single SMTP Command"""
         cmd = data[0:4]
@@ -172,28 +178,28 @@ class SMTPServerEngine:
             keep = 0
         elif cmd == "MAIL":
             if self.state != SMTPServerEngine.ST_HELO:
-                return ("503 Bad command sequence", 1)
+                return "503 Bad command sequence", 1
             self.state = SMTPServerEngine.ST_MAIL
             rv = self.impl.mail_from(data[5:])
         elif cmd == "RCPT":
             if (self.state != SMTPServerEngine.ST_MAIL) and \
                (self.state != SMTPServerEngine.ST_RCPT):
-                return ("503 Bad command sequence", 1)
+                return "503 Bad command sequence", 1
             self.state = SMTPServerEngine.ST_RCPT
             rv = self.impl.rcpt_to(data[5:])
         elif cmd == "DATA":
             if self.state != SMTPServerEngine.ST_RCPT:
-                return ("503 Bad command sequence", 1)
+                return "503 Bad command sequence", 1
             self.state = SMTPServerEngine.ST_DATA
             self.data_accum = ""
-            return ("354 OK, Enter data, terminated with a \\r\\n.\\r\\n", 1)
+            return "354 OK, Enter data, terminated with a \\r\\n.\\r\\n", 1
         else:
-            return ("505 Eh? WTF was that?", 1)
+            return "505 Eh? WTF was that?", 1
 
         if rv:
-            return (rv, keep)
+            return rv, keep
         else:
-            return("250 OK", keep)
+            return "250 OK", keep
 
     def do_data(self, data):
         """
@@ -218,17 +224,17 @@ class SMTPServer:
     A single threaded SMTP Server connection manager. Listens for
     incoming SMTP connections on a given port. For each connection,
     the SMTPServerEngine is chugged, passing the given instance of
-    SMTPServerInterface. 
+    SMTPServerInterface.
     """
-    
+
     def __init__(self, port):
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self._socket.bind(("localhost", port))
+        self._socket.bind(("127.0.0.1", port))
         self._socket_service = None
 
     def serve(self, impl):
-        while ( self._resume ):
+        while self._resume:
             try:
                 nsd = self._socket.accept()
             except socket.error:
@@ -240,14 +246,14 @@ class SMTPServer:
 
     def start(self):
         self._socket.listen(1)
-        self._resume = True        
-        
+        self._resume = True
+
     def stop(self):
         self._resume = False
-        
+
     def terminate(self):
         if self._socket_service:
-            # force the blocking socket to stop waiting for data 
+            # force the blocking socket to stop waiting for data
             try:
                 #self._socket_service.shutdown(2)
                 self._socket_service.close()
@@ -260,6 +266,7 @@ class SMTPServer:
             self._socket.close()
             self._socket = None
 
+
 class SMTPServerStore(SMTPServerInterface):
     """
     Simple store for SMTP data
@@ -270,14 +277,14 @@ class SMTPServerStore(SMTPServerInterface):
 
     def helo(self, args):
         self.reset(None)
-    
+
     def mail_from(self, args):
         if args.lower().startswith('from:'):
-            self.sender = strip_address(args[5:].replace('\r\n','').strip())
-        
+            self.sender = strip_address(args[5:].replace('\r\n', '').strip())
+
     def rcpt_to(self, args):
         if args.lower().startswith('to:'):
-            rcpt = args[3:].replace('\r\n','').strip()
+            rcpt = args[3:].replace('\r\n', '').strip()
             self.recipients.append(strip_address(rcpt))
 
     def data(self, args):
@@ -290,7 +297,7 @@ class SMTPServerStore(SMTPServerInterface):
         self.sender = None
         self.recipients = []
         self.message = None
-        
+
 
 class SMTPThreadedServer(threading.Thread):
     """
@@ -300,26 +307,26 @@ class SMTPThreadedServer(threading.Thread):
     def __init__(self, port):
         self.port = port
         self.server = SMTPServer(port)
-        self.store  = SMTPServerStore()
+        self.store = SMTPServerStore()
         threading.Thread.__init__(self)
-      
+
     def run(self):
         # run from within the SMTP server thread
-        self.server.serve(impl = self.store)
+        self.server.serve(impl=self.store)
 
     def start(self):
         # run from the main thread
         self.server.start()
         threading.Thread.start(self)
-        
+
     def stop(self):
         # run from the main thread
         self.server.stop()
         # send a message to make the SMTP server quit gracefully
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         try:
-            s.connect(('localhost', self.port))
-            r = s.send("QUIT\r\n");
+            s.connect(('127.0.0.1', self.port))
+            r = s.send("QUIT\r\n")
         except socket.error:
             pass
         s.close()
@@ -336,10 +343,11 @@ class SMTPThreadedServer(threading.Thread):
 
     def get_message(self):
         return self.store.message
-        
+
     def cleanup(self):
         self.store.reset(None)
 
+
 def smtp_address(fulladdr):
     mo = email_re.search(fulladdr)
     if mo:
@@ -348,30 +356,32 @@ def smtp_address(fulladdr):
         return fulladdr[start+1:-1]
     return fulladdr
 
+
 def decode_header(header):
     """ Decode a MIME-encoded header value """
     mo = header_re.match(header)
     # header does not seem to be MIME-encoded
     if not mo:
         return header
-    # attempts to decode the hedear, 
-    # following the specified MIME endoding and charset
+    # attempts to decode the header,
+    # following the specified MIME encoding and charset
     try:
-        encoding = mo.group('code').lower() 
-        if encoding  == 'q':
+        encoding = mo.group('code').lower()
+        if encoding == 'q':
             val = quopri.decodestring(mo.group('value'), header=True)
         elif encoding == 'b':
             val = base64.decodestring(mo.group('value'))
         else:
-            raise AssertionError, "unsupported encoding: %s" % encoding
+            raise AssertionError("unsupported encoding: %s" % encoding)
         header = unicode(val, mo.group('charset'))
     except Exception, e:
-        raise AssertionError, e
+        raise AssertionError(e)
     return header
 
+
 def parse_smtp_message(msg):
     """ Split a SMTP message into its headers and body.
-        Returns a (headers, body) tuple 
+        Returns a (headers, body) tuple
         We do not use the email/MIME Python facilities here
         as they may accept invalid RFC822 data, or data we do not
         want to support nor generate """
@@ -381,32 +391,32 @@ def parse_smtp_message(msg):
     # last line does not contain the final line ending
     msg += '\r\n'
     for line in msg.splitlines(True):
-        if body != None:
+        if body is not None:
             # append current line to the body
             if line[-2] == CR:
                 body += line[0:-2]
                 body += '\n'
             else:
-                raise AssertionError, "body misses CRLF: %s (0x%x)" \
-                                      % (line, ord(line[-1]))
+                raise AssertionError("body misses CRLF: %s (0x%x)"
+                                     % (line, ord(line[-1])))
         else:
             if line[-2] != CR:
                 # RFC822 requires CRLF at end of field line
-                raise AssertionError, "header field misses CRLF: %s (0x%x)" \
-                                      % (line, ord(line[-1]))
+                raise AssertionError("header field misses CRLF: %s (0x%x)"
+                                     % (line, ord(line[-1])))
             # discards CR
             line = line[0:-2]
             if line.strip() == '':
                 # end of headers, body starts
-                body = '' 
+                body = ''
             else:
                 val = None
                 if line[0] in ' \t':
-                    # continution of the previous line
+                    # continuation of the previous line
                     if not lh:
                         # unexpected multiline
-                        raise AssertionError, \
-                             "unexpected folded line: %s" % line
+                        raise AssertionError("unexpected folded line: %s"
+                                             % line)
                     val = decode_header(line.strip(' \t'))
                     # appends the current line to the previous one
                     if not isinstance(headers[lh], tuple):
@@ -415,16 +425,54 @@ def parse_smtp_message(msg):
                         headers[lh][-1] = headers[lh][-1] + val
                 else:
                     # splits header name from value
-                    (h,v) = line.split(':',1)
+                    (h, v) = line.split(':', 1)
                     val = decode_header(v.strip())
-                    if headers.has_key(h):
+                    if h in headers:
                         if isinstance(headers[h], tuple):
                             headers[h] += val
                         else:
                             headers[h] = (headers[h], val)
                     else:
                         headers[h] = val
-                    # stores the last header (for multilines headers)
+                    # stores the last header (for multi-line headers)
                     lh = h
     # returns the headers and the message body
-    return (headers, body)
+    return headers, body
+
+
+class SendmailEmailSenderTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+
+    def test_sendmail_path_not_found_raises(self):
+        sender = SendmailEmailSender(self.env)
+        self.env.config.set('notification', 'sendmail_path',
+                            os.path.join(os.path.dirname(__file__),
+                                         'sendmail'))
+        self.assertRaises(ConfigurationError, sender.send,
+                          'admin@domain.com', ['foo@domain.com'], "")
+
+
+class SmtpEmailSenderTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+
+    def test_smtp_server_not_found_raises(self):
+        sender = SmtpEmailSender(self.env)
+        self.env.config.set('notification', 'smtp_server', 'localhost')
+        self.env.config.set('notification', 'smtp_port', '65536')
+        self.assertRaises(ConfigurationError, sender.send,
+                          'admin@domain.com', ['foo@domain.com'], "")
+
+
+def suite():
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(SendmailEmailSenderTestCase))
+    suite.addTest(unittest.makeSuite(SmtpEmailSenderTestCase))
+    return suite
+
+
+if __name__ == '__main__':
+    unittest.main(defaultTest='suite')
