@@ -40,9 +40,9 @@ class ITicketActionController(Interface):
 
     def get_ticket_actions(req, ticket):
         """Return an iterable of `(weight, action)` tuples corresponding to
-        the actions that are contributed by this component.
-        That list may vary given the current state of the ticket and the
-        actual request parameter.
+        the actions that are contributed by this component. The list is
+        dependent on the current state of the ticket and the actual request
+        parameter.
 
         `action` is a key used to identify that particular action.
         (note that 'history' and 'diff' are reserved and should not be used
@@ -52,7 +52,8 @@ class ITicketActionController(Interface):
         integer weight. The first action in the list is used as the default
         action.
 
-        When in doubt, use a weight of 0."""
+        When in doubt, use a weight of 0.
+        """
 
     def get_all_status():
         """Returns an iterable of all the possible values for the ''status''
@@ -231,8 +232,8 @@ class TicketSystem(Component):
         (''since 0.11'').""")
 
     def __init__(self):
-        self.log.debug('action controllers for ticket workflow: %r' %
-                [c.__class__.__name__ for c in self.action_controllers])
+        self.log.debug('action controllers for ticket workflow: %r',
+                       [c.__class__.__name__ for c in self.action_controllers])
 
     # Public API
 
@@ -299,9 +300,8 @@ class TicketSystem(Component):
 
         # Owner field, by default text but can be changed dynamically
         # into a drop-down depending on configuration (restrict_owner=true)
-        field = {'name': 'owner', 'label': N_('Owner')}
-        field['type'] = 'text'
-        fields.append(field)
+        fields.append({'name': 'owner', 'type': 'text',
+                       'label': N_('Owner')})
 
         # Description
         fields.append({'name': 'description', 'type': 'textarea',
@@ -412,30 +412,43 @@ class TicketSystem(Component):
         """
         if self.restrict_owner:
             field['type'] = 'select'
-            possible_owners = []
+            allowed_owners = self.get_allowed_owners(ticket)
+            allowed_owners.insert(0, '< default >')
+            field['options'] = allowed_owners
+            field['optional'] = True
+
+    def get_allowed_owners(self, ticket=None):
+        """Returns a list of permitted ticket owners (those possessing the
+        TICKET_MODIFY permission). Returns `None` if the option `[ticket]`
+        `restrict_owner` is `False`.
+
+        If `ticket` is not `None`, fine-grained permission checks are used
+        to determine the allowed owners for the specified resource.
+
+        :since: 1.0.3
+        """
+        if self.restrict_owner:
+            allowed_owners = []
             for user in PermissionSystem(self.env) \
-                    .get_users_with_permission('TICKET_MODIFY'):
+                        .get_users_with_permission('TICKET_MODIFY'):
                 if not ticket or \
                         'TICKET_MODIFY' in PermissionCache(self.env, user,
                                                            ticket.resource):
-                    possible_owners.append(user)
-            possible_owners.sort()
-            possible_owners.insert(0, '< default >')
-            field['options'] = possible_owners
-            field['optional'] = True
+                    allowed_owners.append(user)
+            allowed_owners.sort()
+            return allowed_owners
 
     # IPermissionRequestor methods
 
     def get_permission_actions(self):
         return ['TICKET_APPEND', 'TICKET_CREATE', 'TICKET_CHGPROP',
                 'TICKET_VIEW', 'TICKET_EDIT_CC', 'TICKET_EDIT_DESCRIPTION',
-                'TICKET_EDIT_COMMENT', 'TICKET_BATCH_MODIFY',
+                'TICKET_EDIT_COMMENT',
                 ('TICKET_MODIFY', ['TICKET_APPEND', 'TICKET_CHGPROP']),
                 ('TICKET_ADMIN', ['TICKET_CREATE', 'TICKET_MODIFY',
                                   'TICKET_VIEW', 'TICKET_EDIT_CC',
                                   'TICKET_EDIT_DESCRIPTION',
-                                  'TICKET_EDIT_COMMENT',
-                                  'TICKET_BATCH_MODIFY'])]
+                                  'TICKET_EDIT_COMMENT'])]
 
     # IWikiSyntaxProvider methods
 
@@ -584,13 +597,16 @@ class TicketSystem(Component):
         >>> resource_exists(env, t.resource)
         True
         """
-        if self.env.db_query("SELECT id FROM ticket WHERE id=%s",
-                             (resource.id,)):
+        try:
+            id_ = int(resource.id)
+        except (TypeError, ValueError):
+            return False
+        if self.env.db_query("SELECT id FROM ticket WHERE id=%s", (id_,)):
             if resource.version is None:
                 return True
             revcount = self.env.db_query("""
                 SELECT count(DISTINCT time) FROM ticket_change WHERE ticket=%s
-                """, (resource.id,))
+                """, (id_,))
             return revcount[0][0] >= resource.version
         else:
             return False
