@@ -11,23 +11,32 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
+from __future__ import with_statement
+
+from datetime import datetime, timedelta
+import difflib
+import re
+import unittest
+
+from trac.mimeview.api import Mimeview
 from trac.test import Mock, EnvironmentStub, MockPerm, locale_en
-from trac.ticket.model import Ticket
+from trac.ticket.api import TicketSystem
+from trac.ticket.model import Milestone, Severity, Ticket, Version
 from trac.ticket.query import Query, QueryModule, TicketQueryMacro
 from trac.util.datefmt import utc
+from trac.web.api import arg_list_to_args, parse_arg_list
 from trac.web.chrome import web_context
 from trac.web.href import Href
 from trac.wiki.formatter import LinkFormatter
 from trac.wiki.tests import formatter
 
-import unittest
-import difflib
-
 # Note: we don't want to replicate 1:1 all the SQL dialect abstraction
 #       methods from the trac.db layer here.
 
 class QueryTestCase(unittest.TestCase):
 
+    n_tickets = 10
+
     def prettifySQL(self, sql):
         """Returns a prettified version of the SQL as a list of lines to help
         in creating a useful diff between two SQL statements."""
@@ -49,10 +58,62 @@ class QueryTestCase(unittest.TestCase):
         self.env = EnvironmentStub(default_data=True)
         self.req = Mock(href=self.env.href, authname='anonymous', tz=utc,
                         locale=locale_en, lc_time=locale_en)
+        self.tktids = self._insert_tickets(
+            owner=[None, '', 'someone', 'someone_else', 'none'],
+            status=[None, '', 'new', 'assigned', 'reopened', 'closed'],
+            priority=[None, '', 'blocker', 'critical', 'major', 'minor',
+                      'trivial'],
+            milestone=[None, '', 'milestone1', 'milestone2'],
+            version=[None, '', '0.0', 'version1', '1.0', '2.0'],
+            keywords=[None, '', 'foo', 'bar', 'baz', 'foo bar', 'bar baz',
+                      'foo baz', 'foo bar baz'])
+        dt = datetime(2008, 7, 1, 12, tzinfo=utc)
+        with self.env.db_transaction:
+            for name in ('milestone1', 'milestone2'):
+                milestone = Milestone(self.env, name)
+                milestone.due = dt
+                milestone.update()
+            for name in ('1.0', '2.0'):
+                version = Version(self.env, name)
+                version.time = dt
+                version.update()
+            for name in ('urgent', 'high', 'medium'):
+                severity = Severity(self.env)
+                severity.name = name
+                severity.insert()
+        tktsys = TicketSystem(self.env)
+        tktsys.reset_ticket_fields()
+        del tktsys.custom_fields
 
     def tearDown(self):
         self.env.reset_db()
 
+    def _insert_tickets(self, owner, status, priority, milestone, version,
+                        keywords):
+        when = datetime(2008, 7, 1, 12, 34, 56, 987654, utc)
+        with self.env.db_transaction:
+            ids = []
+            for idx in xrange(self.n_tickets):
+                t = Ticket(self.env)
+                t['summary'] = 'Summary %d' % idx
+                t['owner'] = owner[idx % len(owner)]
+                t['status'] = status[idx % len(status)]
+                t['priority'] = priority[idx % len(priority)]
+                t['milestone'] = milestone[idx % len(milestone)]
+                t['version'] = version[idx % len(version)]
+                t['keywords'] = keywords[idx % len(keywords)]
+                ids.append(t.insert(when=when + timedelta(days=idx * 10)))
+                t.save_changes(comment='...',
+                               when=when + timedelta(days=idx * 10 + 1))
+        return ids
+
+    def _update_tickets(self, name, values):
+        with self.env.db_transaction:
+            for idx, tktid in enumerate(self.tktids):
+                t = Ticket(self.env, tktid)
+                t[name] = values[idx % len(values)]
+                t.save_changes()
+
     def test_all_ordered_by_id(self):
         query = Query(self.env, order='id')
         sql, args = query.get_sql()
@@ -63,6 +124,8 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(self.n_tickets, len(tickets))
+        self.assertTrue(tickets[0]['id'] < tickets[-1]['id'])
 
     def test_all_ordered_by_id_desc(self):
         query = Query(self.env, order='id', desc=1)
@@ -74,6 +137,8 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0 DESC,t.id DESC""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(self.n_tickets, len(tickets))
+        self.assertTrue(tickets[0]['id'] > tickets[-1]['id'])
 
     def test_all_ordered_by_id_verbose(self):
         query = Query(self.env, order='id', verbose=1)
@@ -85,6 +150,7 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(self.n_tickets, len(tickets))
 
     def test_all_ordered_by_id_from_unicode(self):
         query = Query.from_string(self.env, u'order=id')
@@ -96,6 +162,7 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(self.n_tickets, len(tickets))
 
     def test_all_ordered_by_priority(self):
         query = Query(self.env) # priority is default order
@@ -108,6 +175,9 @@ ORDER BY COALESCE(priority.value,'')='',%(cast_priority)s,t.id""" % {
           'cast_priority': self.env.get_read_db().cast('priority.value', 'int')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['blocker', 'blocker', 'critical', 'major', 'minor',
+                          'trivial', '', '', '', ''],
+                         [t['priority'] for t in tickets])
 
     def test_all_ordered_by_priority_desc(self):
         query = Query(self.env, desc=1) # priority is default order
@@ -120,6 +190,9 @@ ORDER BY COALESCE(priority.value,'')='' DESC,%(cast_priority)s DESC,t.id""" % {
           'cast_priority': self.env.get_read_db().cast('priority.value', 'int')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', '', '', 'trivial', 'minor', 'major',
+                          'critical', 'blocker', 'blocker'],
+                         [t['priority'] for t in tickets])
 
     def test_all_ordered_by_version(self):
         query = Query(self.env, order='version')
@@ -132,6 +205,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.version,'')='',COALESCE(version.time,0)=0,version.time,t.version,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['1.0', '2.0', '0.0', '0.0', 'version1', 'version1',
+                          '', '', '', ''],
+                         [t['version'] for t in tickets])
 
     def test_all_ordered_by_version_desc(self):
         query = Query(self.env, order='version', desc=1)
@@ -144,6 +220,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.version,'')='' DESC,COALESCE(version.time,0)=0 DESC,version.time DESC,t.version DESC,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', '', '', 'version1', 'version1', '0.0', '0.0',
+                          '2.0', '1.0'],
+                         [t['version'] for t in tickets])
 
     def test_constrained_by_milestone(self):
         query = Query.from_string(self.env, 'milestone=milestone1', order='id')
@@ -156,6 +235,8 @@ WHERE ((COALESCE(t.milestone,'')=%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['milestone1', 'milestone1'],
+                         [t['milestone'] for t in tickets])
 
     def test_all_grouped_by_milestone(self):
         query = Query(self.env, order='id', group='milestone')
@@ -168,6 +249,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.milestone,'')='',COALESCE(milestone.completed,0)=0,milestone.completed,COALESCE(milestone.due,0)=0,milestone.due,t.milestone,COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['milestone1', 'milestone1', 'milestone2',
+                          'milestone2', '', '', '', '', '', ''],
+                         [t['milestone'] for t in tickets])
 
     def test_all_grouped_by_milestone_desc(self):
         query = Query(self.env, order='id', group='milestone', groupdesc=1)
@@ -180,6 +264,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.milestone,'')='' DESC,COALESCE(milestone.completed,0)=0 DESC,milestone.completed DESC,COALESCE(milestone.due,0)=0 DESC,milestone.due DESC,t.milestone DESC,COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', '', '', '', '', 'milestone2', 'milestone2',
+                          'milestone1', 'milestone1'],
+                         [t['milestone'] for t in tickets])
 
     def test_grouped_by_priority(self):
         query = Query(self.env, group='priority')
@@ -192,6 +279,9 @@ ORDER BY COALESCE(priority.value,'')='',%(cast_priority)s,t.id""" % {
           'cast_priority': self.env.get_read_db().cast('priority.value', 'int')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['blocker', 'blocker', 'critical', 'major', 'minor',
+                          'trivial', '', '', '', ''],
+                         [t['priority'] for t in tickets])
 
     def test_constrained_by_milestone_not(self):
         query = Query.from_string(self.env, 'milestone!=milestone1', order='id')
@@ -204,6 +294,8 @@ WHERE ((COALESCE(t.milestone,'')!=%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'milestone2', '', '', 'milestone2', '', ''],
+                         [t['milestone'] for t in tickets])
 
     def test_constrained_by_status(self):
         query = Query.from_string(self.env, 'status=new|assigned|reopened',
@@ -217,6 +309,8 @@ WHERE (COALESCE(t.status,'') IN (%s,%s,%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['new', 'assigned', 'reopened'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['new', 'assigned', 'reopened', 'new', 'assigned'],
+                         [t['status'] for t in tickets])
 
     def test_constrained_by_owner_containing(self):
         query = Query.from_string(self.env, 'owner~=someone', order='id')
@@ -229,6 +323,9 @@ WHERE ((COALESCE(t.owner,'') %(like)s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%someone%'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone_else', 'someone',
+                          'someone_else'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_owner_not_containing(self):
         query = Query.from_string(self.env, 'owner!~=someone', order='id')
@@ -241,6 +338,8 @@ WHERE ((COALESCE(t.owner,'') NOT %(like)s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%someone%'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'none', '', '', 'none'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_owner_beginswith(self):
         query = Query.from_string(self.env, 'owner^=someone', order='id')
@@ -253,6 +352,9 @@ WHERE ((COALESCE(t.owner,'') %(like)s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['someone%'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone_else', 'someone',
+                          'someone_else'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_owner_endswith(self):
         query = Query.from_string(self.env, 'owner$=someone', order='id')
@@ -265,40 +367,41 @@ WHERE ((COALESCE(t.owner,'') %(like)s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%someone'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone'], [t['owner'] for t in tickets])
 
     def test_constrained_by_custom_field(self):
         self.env.config.set('ticket-custom', 'foo', 'text')
+        self._update_tickets('foo', [None, '', 'something'])
         query = Query.from_string(self.env, 'foo=something', order='id')
         sql, args = query.get_sql()
-        foo = self.env.get_read_db().quote('foo')
         self.assertEqualSQL(sql,
-"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,t.%s AS %s
-FROM (
-  SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,
-  (SELECT c.value FROM ticket_custom c WHERE c.ticket=t.id AND c.name='foo') AS %s
-  FROM ticket AS t) AS t
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,%(foo)s.value AS %(foo)s
+FROM ticket AS t
+  LEFT OUTER JOIN ticket_custom AS %(foo)s ON (%(foo)s.ticket=t.id AND %(foo)s.name='foo')
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE ((COALESCE(t.%s,'')=%%s))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % ((foo,) * 4))
+WHERE ((COALESCE(%(foo)s.value,'')=%%s))
+ORDER BY COALESCE(t.id,0)=0,t.id"""
+        % {'foo': self.env.get_read_db().quote('foo')})
         self.assertEqual(['something'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['something'] * 3, [t['foo'] for t in tickets])
 
     def test_grouped_by_custom_field(self):
         self.env.config.set('ticket-custom', 'foo', 'text')
+        self._update_tickets('foo', [None, '', 'something'])
         query = Query(self.env, group='foo', order='id')
         sql, args = query.get_sql()
-        foo = self.env.get_read_db().quote('foo')
         self.assertEqualSQL(sql,
-"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,t.%s AS %s
-FROM (
-  SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,
-  (SELECT c.value FROM ticket_custom c WHERE c.ticket=t.id AND c.name='foo') AS %s
-  FROM ticket AS t) AS t
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,%(foo)s.value AS %(foo)s
+FROM ticket AS t
+  LEFT OUTER JOIN ticket_custom AS %(foo)s ON (%(foo)s.ticket=t.id AND %(foo)s.name='foo')
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-ORDER BY COALESCE(t.%s,'')='',t.%s,COALESCE(t.id,0)=0,t.id""" %
-        ((foo,) * 5))
+ORDER BY COALESCE(%(foo)s.value,'')='',%(foo)s.value,COALESCE(t.id,0)=0,t.id"""
+        % {'foo': self.env.get_read_db().quote('foo')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['something'] * 3 + [''] * 7,
+                         [t['foo'] for t in tickets])
 
     def test_constrained_by_id_ranges(self):
         query = Query.from_string(self.env, 'id=42,44,51-55&order=id')
@@ -335,6 +438,65 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(1, len(tickets))
         self.assertEqual(ticket.id, tickets[0]['id'])
 
+    def _get_join_tables(self, sql):
+        return sorted(match.group(1)
+                      for match in re.finditer(r'\bLEFT OUTER JOIN (\w+)',
+                                               sql))
+
+    def test_query_using_joins(self):
+        fields = ['col_%02d' % i for i in xrange(100)]
+        for f in fields:
+            self.env.config.set('ticket-custom', f, 'text')
+        with self.env.db_transaction as db:
+            quoted_cols = dict((f, db.quote(f)) for f in fields)
+            ticket = Ticket(self.env)
+            ticket['reporter'] = 'joe'
+            ticket['summary'] = 'Foo'
+            for idx, f in enumerate(fields):
+                ticket[f] = '%d.%s' % (idx, f)
+            ticket.insert()
+
+        query = Query.from_string(
+            self.env, 'col_12=12.col_12&' +
+                      'order=resolution&group=severity&col=id&col=summary' +
+                      ''.join('&col=col_%02d' % idx for idx in xrange(28)))
+        sql, args = query.get_sql()
+        self.assertEqual(['enum'] * 3 + ['ticket_custom'] * 28,
+                         self._get_join_tables(sql))
+        tickets = query.execute(self.req)
+        self.assertEqual(1, len(tickets))
+
+        query = Query.from_string(
+            self.env, 'col_12=12.col_12&' +
+                      'order=milestone&group=version&col=id&col=summary' +
+                      ''.join('&col=col_%02d' % idx for idx in xrange(28)))
+        sql, args = query.get_sql()
+        self.assertEqual(['enum', 'milestone'] + ['ticket_custom'] * 28 +
+                         ['version'],
+                         self._get_join_tables(sql))
+        tickets = query.execute(self.req)
+        self.assertEqual(1, len(tickets))
+
+        query = Query.from_string(
+            self.env, 'col_12=12.col_12&' +
+                      'order=resolution&group=severity&col=id&col=summary' +
+                      ''.join('&col=col_%02d' % idx for idx in xrange(29)))
+        sql, args = query.get_sql()
+        self.assertEqual(['enum'] * 3, self._get_join_tables(sql))
+        tickets = query.execute(self.req)
+        self.assertEqual(ticket.id, tickets[0]['id'])
+        self.assertEqual(1, len(tickets))
+
+        query = Query.from_string(
+            self.env, 'col_12=12.col_12&' +
+                      'order=milestone&group=version&col=id&col=summary' +
+                      ''.join('&col=col_%02d' % idx for idx in xrange(29)))
+        sql, args = query.get_sql()
+        self.assertEqual(['enum', 'milestone', 'version'],
+                         self._get_join_tables(sql))
+        tickets = query.execute(self.req)
+        self.assertEqual(1, len(tickets))
+
     def test_too_many_custom_fields(self):
         fields = ['col_%02d' % i for i in xrange(100)]
         for f in fields:
@@ -372,6 +534,9 @@ WHERE (COALESCE(t.owner,'') IN (%s,%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone', 'someone_else'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone_else', 'someone',
+                          'someone_else'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_multiple_owners_not(self):
         query = Query.from_string(self.env, 'owner!=someone|someone_else',
@@ -385,6 +550,8 @@ WHERE (COALESCE(t.owner,'') NOT IN (%s,%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone', 'someone_else'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'none', '', '', 'none'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_multiple_owners_contain(self):
         query = Query.from_string(self.env, 'owner~=someone|someone_else',
@@ -398,6 +565,65 @@ FROM ticket AS t
 WHERE ((COALESCE(t.owner,'') %(like)s OR COALESCE(t.owner,'') %(like)s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone_else', 'someone',
+                          'someone_else'],
+                         [t['owner'] for t in tickets])
+
+    def test_constrained_by_an_empty_value(self):
+        query = Query.from_string(self.env, 'owner=', order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((COALESCE(t.owner,'')=%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual([''], args)
+        tickets = query.execute(self.req)
+        self.assertEqual(['', '', '', ''], [t['owner'] for t in tickets])
+
+    def test_constrained_by_an_empty_value_not(self):
+        query = Query.from_string(self.env, 'owner!=', order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((COALESCE(t.owner,'')!=%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual([''], args)
+        tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone_else', 'none', 'someone',
+                          'someone_else', 'none'],
+                         [t['owner'] for t in tickets])
+
+    def test_constrained_by_empty_values(self):
+        query = Query.from_string(self.env, 'owner=|', order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE (COALESCE(t.owner,'') IN (%s,%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual(['', ''], args)
+        tickets = query.execute(self.req)
+        self.assertEqual(['', '', '', ''], [t['owner'] for t in tickets])
+
+    def test_constrained_by_empty_values_not(self):
+        query = Query.from_string(self.env, 'owner!=|', order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE (COALESCE(t.owner,'') NOT IN (%s,%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual(['', ''], args)
+        tickets = query.execute(self.req)
+        self.assertEqual(['someone', 'someone_else', 'none', 'someone',
+                          'someone_else', 'none'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_empty_value_contains(self):
         query = Query.from_string(self.env, 'owner~=|', order='id')
@@ -409,6 +635,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'someone', 'someone_else', 'none', '', '',
+                          'someone', 'someone_else', 'none'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_empty_value_startswith(self):
         query = Query.from_string(self.env, 'owner^=|', order='id')
@@ -420,6 +649,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'someone', 'someone_else', 'none', '', '',
+                          'someone', 'someone_else', 'none'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_empty_value_endswith(self):
         query = Query.from_string(self.env, 'owner$=|', order='id')
@@ -431,6 +663,9 @@ FROM ticket AS t
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'someone', 'someone_else', 'none', '', '',
+                          'someone', 'someone_else', 'none'],
+                         [t['owner'] for t in tickets])
 
     def test_constrained_by_time_range(self):
         query = Query.from_string(self.env, 'created=2008-08-01..2008-09-01', order='id')
@@ -444,6 +679,10 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {
           'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1217548800000000L, 1220227200000000L], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['2008-08-10T12:34:56.987654+00:00',
+                          '2008-08-20T12:34:56.987654+00:00',
+                          '2008-08-30T12:34:56.987654+00:00'],
+                         [t['time'].isoformat() for t in tickets])
 
     def test_constrained_by_time_range_exclusion(self):
         query = Query.from_string(self.env, 'created!=2008-08-01..2008-09-01', order='id')
@@ -457,6 +696,14 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {
           'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1217548800000000L, 1220227200000000L], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['2008-07-01T12:34:56.987654+00:00',
+                          '2008-07-11T12:34:56.987654+00:00',
+                          '2008-07-21T12:34:56.987654+00:00',
+                          '2008-07-31T12:34:56.987654+00:00',
+                          '2008-09-09T12:34:56.987654+00:00',
+                          '2008-09-19T12:34:56.987654+00:00',
+                          '2008-09-29T12:34:56.987654+00:00'],
+                         [t['time'].isoformat() for t in tickets])
 
     def test_constrained_by_time_range_open_right(self):
         query = Query.from_string(self.env, 'created=2008-08-01..', order='id')
@@ -470,6 +717,13 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {
           'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1217548800000000L], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['2008-08-10T12:34:56.987654+00:00',
+                          '2008-08-20T12:34:56.987654+00:00',
+                          '2008-08-30T12:34:56.987654+00:00',
+                          '2008-09-09T12:34:56.987654+00:00',
+                          '2008-09-19T12:34:56.987654+00:00',
+                          '2008-09-29T12:34:56.987654+00:00'],
+                         [t['time'].isoformat() for t in tickets])
 
     def test_constrained_by_time_range_open_left(self):
         query = Query.from_string(self.env, 'created=..2008-09-01', order='id')
@@ -483,6 +737,14 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {
           'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1220227200000000L], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['2008-07-01T12:34:56.987654+00:00',
+                          '2008-07-11T12:34:56.987654+00:00',
+                          '2008-07-21T12:34:56.987654+00:00',
+                          '2008-07-31T12:34:56.987654+00:00',
+                          '2008-08-10T12:34:56.987654+00:00',
+                          '2008-08-20T12:34:56.987654+00:00',
+                          '2008-08-30T12:34:56.987654+00:00'],
+                         [t['time'].isoformat() for t in tickets])
 
     def test_constrained_by_time_range_modified(self):
         query = Query.from_string(self.env, 'modified=2008-08-01..2008-09-01', order='id')
@@ -496,6 +758,11 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {
           'cast_changetime': self.env.get_read_db().cast('t.changetime', 'int64')})
         self.assertEqual([1217548800000000L, 1220227200000000L], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['2008-08-01T12:34:56.987654+00:00',
+                          '2008-08-11T12:34:56.987654+00:00',
+                          '2008-08-21T12:34:56.987654+00:00',
+                          '2008-08-31T12:34:56.987654+00:00'],
+                         [t['changetime'].isoformat() for t in tickets])
 
     def test_constrained_by_keywords(self):
         query = Query.from_string(self.env, 'keywords~=foo -bar baz',
@@ -509,6 +776,23 @@ WHERE (((COALESCE(t.keywords,'') %(like)s AND COALESCE(t.keywords,'') NOT %(like
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%foo%', '%bar%', '%baz%'], args)
         tickets = query.execute(self.req)
+        self.assertEqual(['foo baz'], [t['keywords'] for t in tickets])
+
+    def test_constrained_by_keywords_not(self):
+        query = Query.from_string(self.env, 'keywords!~=foo -bar baz',
+                                  order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.keywords AS keywords,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((NOT (COALESCE(t.keywords,'') %(like)s AND COALESCE(t.keywords,'') NOT %(like)s AND COALESCE(t.keywords,'') %(like)s)))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
+        self.assertEqual(['%foo%', '%bar%', '%baz%'], args)
+        tickets = query.execute(self.req)
+        self.assertEqual(['', '', 'foo', 'bar', 'baz', 'foo bar', 'bar baz',
+                          'foo bar baz', ''],
+                         [t['keywords'] for t in tickets])
 
     def test_constrained_by_milestone_or_version(self):
         query = Query.from_string(self.env, 'milestone=milestone1&or&version=version1', order='id')
@@ -521,6 +805,11 @@ WHERE ((COALESCE(t.milestone,'')=%s)) OR ((COALESCE(t.version,'')=%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1', 'version1'], args)
         tickets = query.execute(self.req)
+        self.assertEqual([('milestone1', '0.0'),
+                          ('milestone2', 'version1'),
+                          ('milestone1', ''),
+                          ('',           'version1')],
+                         [(t['milestone'], t['version']) for t in tickets])
 
     def test_equal_in_value(self):
         query = Query.from_string(self.env, r'status=this=that&version=version1',
@@ -572,14 +861,14 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         tickets = query.execute(self.req)
 
     def test_csv_escape(self):
-        query = Mock(get_columns=lambda: ['col1'],
+        query = Mock(get_columns=lambda: ['id', 'col1'],
                      execute=lambda r: [{'id': 1,
                                          'col1': 'value, needs escaped'}],
                      time_fields=['time', 'changetime'])
-        content, mimetype = QueryModule(self.env).export_csv(
-                                Mock(href=self.env.href, perm=MockPerm()),
-                                query)
-        self.assertEqual('\xef\xbb\xbfcol1\r\n"value, needs escaped"\r\n',
+        req = Mock(href=self.env.href, perm=MockPerm())
+        content, mimetype, ext = Mimeview(self.env).convert_content(
+            req, 'trac.ticket.Query', query, 'csv')
+        self.assertEqual('\xef\xbb\xbfid,col1\r\n1,"value, needs escaped"\r\n',
                          content)
 
     def test_csv_obfuscation(self):
@@ -589,22 +878,24 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
                 return action != 'EMAIL_VIEW'
             __contains__ = has_permission
 
-        query = Mock(get_columns=lambda: ['owner', 'reporter', 'cc'],
+        query = Mock(get_columns=lambda: ['id', 'owner', 'reporter', 'cc'],
                      execute=lambda r: [{'id': 1,
                                          'owner': 'joe@example.org',
                                          'reporter': 'foo@example.org',
                                          'cc': 'cc1@example.org, cc2'}],
                      time_fields=['time', 'changetime'])
         req = Mock(href=self.env.href, perm=NoEmailView())
-        content, mimetype = QueryModule(self.env).export_csv(req, query)
-        self.assertEqual(u'\uFEFFowner,reporter,cc\r\n'
-                         u'joe@…,foo@…,"cc1@…, cc2"\r\n',
+        content, mimetype, ext = Mimeview(self.env).convert_content(
+            req, 'trac.ticket.Query', query, 'csv')
+        self.assertEqual(u'\uFEFFid,owner,reporter,cc\r\n'
+                         u'1,joe@…,foo@…,"cc1@…, cc2"\r\n',
                          content.decode('utf-8'))
         req = Mock(href=self.env.href, perm=MockPerm())
-        content, mimetype = QueryModule(self.env).export_csv(req, query)
+        content, mimetype, ext = Mimeview(self.env).convert_content(
+            req, 'trac.ticket.Query', query, 'csv')
         self.assertEqual(
-            u'\uFEFFowner,reporter,cc\r\n'
-            u'joe@example.org,foo@example.org,"cc1@example.org, cc2"\r\n',
+            u'\uFEFFid,owner,reporter,cc\r\n'
+            u'1,joe@example.org,foo@example.org,"cc1@example.org, cc2"\r\n',
             content.decode('utf-8'))
 
     def test_template_data(self):
@@ -628,12 +919,25 @@ class QueryLinksTestCase(unittest.TestCase):
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
         self.query_module = QueryModule(self.env)
-        req = Mock(perm=MockPerm(), args={}, href=Href('/'))
-        self.formatter = LinkFormatter(self.env, web_context(req))
+        self.req = Mock(perm=MockPerm(), args={}, arg_list=(), href=Href('/'),
+                        authname='anonymous', chrome={}, session={}, tz=utc,
+                        locale=None, lc_time=None, path_info='/query')
+        self.context = web_context(self.req)
+        self.formatter = LinkFormatter(self.env, self.context)
 
     def tearDown(self):
         self.env.reset_db()
 
+    def _insert_ticket(self, **attrs):
+        attrs.setdefault('reporter', 'joe')
+        attrs.setdefault('summary', 'Summary')
+        attrs.setdefault('status', 'new')
+        ticket = Ticket(self.env)
+        for name, value in attrs.iteritems():
+            ticket[name] = value
+        ticket.insert()
+        return ticket
+
     def _format_link(self, query, label):
         return str(self.query_module._format_link(self.formatter, 'query',
                                                   query, label))
@@ -643,6 +947,38 @@ class QueryLinksTestCase(unittest.TestCase):
                          '<em class="error">[Error: Query filter requires '
                          'field and constraints separated by a "="]</em>')
 
+    def _process_request(self, query_string):
+        self.req.arg_list = parse_arg_list(query_string)
+        self.req.args = arg_list_to_args(self.req.arg_list)
+        self.assertEqual(True, self.query_module.match_request(self.req))
+        template, data, content_type = \
+                self.query_module.process_request(self.req)
+        return data
+
+    def test_duplicated_order_arguments(self):
+        data = self._process_request('order=priority&order=id')
+        self.assertEqual([], data['tickets'])
+        self.assertEqual('priority', data['query'].order)
+
+    def test_duplicated_report_arguments(self):
+        data = self._process_request('report=1&report=2')
+        self.assertEqual([], data['tickets'])
+        self.assertEqual('1', data['query'].id)
+
+    def test_duplicated_group_arguments(self):
+        self._insert_ticket(status='new')
+        self._insert_ticket(status='assigned')
+        data = self._process_request(
+                'group=status&group=status&order=priority')
+        self.assertNotEqual([], data['tickets'])
+        self.assertEqual(set(('new', 'assigned')),
+                         set(t['status'] for t in data['tickets']))
+        self.assertEqual(2, len(data['tickets']))
+        self.assertNotEqual([], data['groups'])
+        self.assertEqual(set(('new', 'assigned')),
+                         set(value for value, tickets in data['groups']))
+        self.assertEqual(2, len(data['groups']))
+
 
 class TicketQueryMacroTestCase(unittest.TestCase):
 
@@ -681,6 +1017,39 @@ class TicketQueryMacroTestCase(unittest.TestCase):
                            'list')
 
 QUERY_TEST_CASES = u"""
+
+============================== TicketQuery
+[[TicketQuery]]
+------------------------------
+<p>
+</p><div>\
+<dl class="wiki compact">\
+<dt><a class="new" href="/ticket/1" title="This is the summary">#1</a></dt>\
+<dd>This is the summary</dd>\
+<dt><a class="assigned" href="/ticket/2" title="This is another summary">#2</a></dt>\
+<dd>This is another summary</dd>\
+<dt><a class="closed" href="/ticket/3" title="This is th third summary">#3</a></dt>\
+<dd>This is th third summary</dd>\
+</dl>\
+</div><p>
+</p>
+------------------------------
+============================== TicketQuery()
+[[TicketQuery()]]
+------------------------------
+<p>
+</p><div>\
+<dl class="wiki compact">\
+<dt><a class="new" href="/ticket/1" title="This is the summary">#1</a></dt>\
+<dd>This is the summary</dd>\
+<dt><a class="assigned" href="/ticket/2" title="This is another summary">#2</a></dt>\
+<dd>This is another summary</dd>\
+<dt><a class="closed" href="/ticket/3" title="This is th third summary">#3</a></dt>\
+<dd>This is th third summary</dd>\
+</dl>\
+</div><p>
+</p>
+------------------------------
 ============================== TicketQuery(format=progress)
 [[TicketQuery(format=progress)]]
 ------------------------------
