@@ -19,6 +19,8 @@
 
 from __future__ import with_statement
 
+from cStringIO import StringIO
+import csv
 import errno
 import functools
 import inspect
@@ -39,7 +41,6 @@ from .compat import any, md5, sha1, sorted
 from .datefmt import to_datetime, to_timestamp, utc
 from .text import exception_to_unicode, to_unicode, getpreferredencoding
 
-# -- req, session and web utils
 
 def get_reporter_id(req, arg_name=None):
     """Get most informative "reporter" identity out of a request.
@@ -64,6 +65,7 @@ def get_reporter_id(req, arg_name=None):
         return '%s <%s>' % (name, email)
     return name or email or req.authname # == 'anonymous'
 
+
 def content_disposition(type=None, filename=None):
     """Generate a properly escaped Content-Disposition header."""
     type = type or ''
@@ -169,15 +171,15 @@ class AtomicFile(object):
     """
     def __init__(self, path, mode='w', bufsize=-1):
         self._file = None
-        self._path = path
-        (dir, name) = os.path.split(path)
-        (fd, self._temp) = tempfile.mkstemp(prefix=name + '-', dir=dir)
+        self._path = os.path.realpath(path)
+        dir, name = os.path.split(self._path)
+        fd, self._temp = tempfile.mkstemp(prefix=name + '-', dir=dir)
         self._file = os.fdopen(fd, mode, bufsize)
 
         # Try to preserve permissions and group ownership, but failure
         # should not be fatal
         try:
-            st = os.stat(path)
+            st = os.stat(self._path)
             if hasattr(os, 'chmod'):
                 os.chmod(self._temp, st.st_mode)
             if hasattr(os, 'chflags') and hasattr(st, 'st_flags'):
@@ -258,6 +260,27 @@ def create_unique_file(path):
             path = '%s.%d%s' % (parts[0], idx, parts[1])
 
 
+if os.name == 'nt':
+    def touch_file(filename):
+        """Update modified time of the given file. The file is created if
+        missing."""
+        # Use f.truncate() to avoid low resolution of GetSystemTime()
+        # on Windows
+        with open(filename, 'ab') as f:
+            stat = os.fstat(f.fileno())
+            f.truncate(stat.st_size)
+else:
+    def touch_file(filename):
+        """Update modified time of the given file. The file is created if
+        missing."""
+        try:
+            os.utime(filename, None)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                with open(filename, 'ab'):
+                    pass
+
+
 def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=False,
                    comment=None):
     """Create a instance of `ZipInfo`.
@@ -303,16 +326,16 @@ def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=Fa
         if not zipinfo.filename.endswith('/'):
             zipinfo.filename += '/'
         zipinfo.compress_type = ZIP_STORED
-        zipinfo.external_attr = 040755 << 16L       # permissions drwxr-xr-x
-        zipinfo.external_attr |= 0x10               # MS-DOS directory flag
+        zipinfo.external_attr = 040755 << 16L        # permissions drwxr-xr-x
+        zipinfo.external_attr |= 0x10                # MS-DOS directory flag
     else:
         zipinfo.compress_type = ZIP_DEFLATED
-        zipinfo.external_attr = 0644 << 16L         # permissions -r-wr--r--
+        zipinfo.external_attr = 0644 << 16L          # permissions -r-wr--r--
         if executable:
-            zipinfo.external_attr |= 0755 << 16L    # -rwxr-xr-x
+            zipinfo.external_attr |= 0755 << 16L     # -rwxr-xr-x
         if symlink:
             zipinfo.compress_type = ZIP_STORED
-            zipinfo.external_attr |= 0120000 << 16L # symlink file type
+            zipinfo.external_attr |= 0120000 << 16L  # symlink file type
 
     if comment:
         zipinfo.comment = comment.encode('utf-8')
@@ -348,13 +371,9 @@ class NaivePopen:
         try:
             self.err = None
             self.errorlevel = os.system(command) >> 8
-            outfd = file(outfile, 'r')
-            self.out = outfd.read()
-            outfd.close()
+            self.out = read_file(outfile)
             if capturestderr:
-                errfd = file(errfile,'r')
-                self.err = errfd.read()
-                errfd.close()
+                self.err = read_file(errfile)
         finally:
             if os.path.isfile(outfile):
                 os.remove(outfile)
@@ -388,7 +407,6 @@ def terminate(process):
         except OSError, e:
             # If the process has already finished and has not been
             # waited for, killing it raises an ESRCH error on Cygwin
-            import errno
             if e.errno != errno.ESRCH:
                 raise
 
@@ -453,8 +471,8 @@ def copytree(src, dst, symlinks=False, skip=[], overwrite=False):
                 errors.extend(err.args[0])
         try:
             shutil.copystat(src, dst)
-        except WindowsError, why:
-            pass # Ignore errors due to limited Windows copystat support
+        except WindowsError:
+            pass  # Ignore errors due to limited Windows copystat support
         except OSError, why:
             errors.append((src, dst, str(why)))
         if errors:
@@ -522,7 +540,6 @@ def get_last_traceback():
 
 
 _egg_path_re = re.compile(r'build/bdist\.[^/]+/egg/(.*)')
-
 def get_lines_from_file(filename, lineno, context=0, globals=None):
     """Return `content` number of lines before and after the specified
     `lineno` from the (source code) file identified by `filename`.
@@ -566,10 +583,10 @@ def get_lines_from_file(filename, lineno, context=0, globals=None):
             break
 
     before = [to_unicode(l.rstrip('\n'), charset)
-                 for l in lines[lbound:lineno]]
+              for l in lines[lbound:lineno]]
     line = to_unicode(lines[lineno].rstrip('\n'), charset)
-    after = [to_unicode(l.rstrip('\n'), charset) \
-                 for l in lines[lineno + 1:ubound]]
+    after = [to_unicode(l.rstrip('\n'), charset)
+             for l in lines[lineno + 1:ubound]]
 
     return before, line, after
 
@@ -639,15 +656,14 @@ def get_doc(obj):
     """
     doc = inspect.getdoc(obj)
     if not doc:
-        return (None, None)
+        return None, None
     doc = to_unicode(doc).split('\n\n', 1)
     summary = doc[0].replace('\n', ' ')
     description = doc[1] if len(doc) > 1 else None
-    return (summary, description)
+    return summary, description
 
 
 _dont_import = frozenset(['__file__', '__name__', '__package__'])
-
 def import_namespace(globals_dict, module_name):
     """Import the namespace of a module into a globals dict.
 
@@ -697,6 +713,7 @@ def get_sources(path):
             pass    # Metadata not found
     return sources
 
+
 def get_pkginfo(dist):
     """Get a dictionary containing package information for a package
 
@@ -721,6 +738,9 @@ def get_pkginfo(dist):
                 resource_name = os.path.normpath(resource_name)
                 return any(resource_name == os.path.normpath(name)
                            for name in dist.get_metadata_lines('SOURCES.txt'))
+            if dist.has_metadata('RECORD'):  # *.dist-info/RECORD
+                reader = csv.reader(StringIO(dist.get_metadata('RECORD')))
+                return any(resource_name == row[0] for row in reader)
             toplevel = resource_name.split('/')[0]
             if dist.has_metadata('top_level.txt'):
                 return toplevel in dist.get_metadata_lines('top_level.txt')
@@ -740,21 +760,25 @@ def get_pkginfo(dist):
         else:
             return {}
     import email
+    from trac.util.translation import _
     attrs = ('author', 'author-email', 'license', 'home-page', 'summary',
              'description', 'version')
     info = {}
     def normalize(attr):
         return attr.lower().replace('-', '_')
+    metadata = 'METADATA' if dist.has_metadata('METADATA') else 'PKG-INFO'
     try:
-        pkginfo = email.message_from_string(dist.get_metadata('PKG-INFO'))
+        pkginfo = email.message_from_string(dist.get_metadata(metadata))
         for attr in [key for key in attrs if key in pkginfo]:
             info[normalize(attr)] = pkginfo[attr]
     except IOError, e:
-        err = 'Failed to read PKG-INFO file for %s: %s' % (dist, e)
+        err = _("Failed to read %(metadata)s file for %(dist)s: %(err)s",
+                metadata=metadata, dist=dist, err=to_unicode(e))
         for attr in attrs:
             info[normalize(attr)] = err
     except email.Errors.MessageError, e:
-        err = 'Failed to parse PKG-INFO file for %s: %s' % (dist, e)
+        err = _("Failed to parse %(metadata)s file for %(dist)s: %(err)s",
+                metadata=metadata, dist=dist, err=to_unicode(e))
         for attr in attrs:
             info[normalize(attr)] = err
     return info
@@ -797,6 +821,7 @@ def hex_entropy(digits=32):
     result = ''.join('%.2x' % ord(v) for v in urandom((digits + 1) // 2))
     return result[:digits] if len(result) > digits else result
 
+
 # Original license for md5crypt:
 # Based on FreeBSD src/lib/libcrypt/crypt.c 1.2
 #
@@ -985,15 +1010,15 @@ class Ranges(object):
         p.sort()
         i = 0
         while i + 1 < len(p):
-            if p[i+1][0]-1 <= p[i][1]: # this item overlaps with the next
+            if p[i+1][0]-1 <= p[i][1]:  # this item overlaps with the next
                 # make the first include the second
                 p[i] = (p[i][0], max(p[i][1], p[i+1][1]))
-                del p[i+1] # delete the second, after adjusting my endpoint
+                del p[i+1]  # delete the second, after adjusting my endpoint
             else:
                 i += 1
         if p:
-            self.a = p[0][0] # min value
-            self.b = p[-1][1] # max value
+            self.a = p[0][0]   # min value
+            self.b = p[-1][1]  # max value
         else:
             self.a = self.b = None
 
@@ -1152,6 +1177,7 @@ def embedded_numbers(s):
     pieces[1::2] = map(int, pieces[1::2])
     return pieces
 
+
 def pairwise(iterable):
     """
     >>> list(pairwise([0, 1, 2, 3]))
@@ -1167,6 +1193,7 @@ def pairwise(iterable):
         pass
     return izip(a, b)
 
+
 def partition(iterable, order=None):
     """
     >>> partition([(1, "a"), (2, "b"), (3, "a")])
@@ -1184,6 +1211,7 @@ def partition(iterable, order=None):
         return result
     return [result[key] for key in order]
 
+
 def as_int(s, default, min=None, max=None):
     """Convert s to an int and limit it to the given range, or return default
     if unsuccessful."""
@@ -1197,6 +1225,7 @@ def as_int(s, default, min=None, max=None):
         value = max
     return value
 
+
 def as_bool(value):
     """Convert the given value to a `bool`.
 
@@ -1214,11 +1243,19 @@ def as_bool(value):
     except (TypeError, ValueError):
         return False
 
+
 def pathjoin(*args):
     """Strip `/` from the arguments and join them with a single `/`."""
     return '/'.join(filter(None, (each.strip('/') for each in args if each)))
 
 
+def to_list(splittable, sep=','):
+    """Split a string at `sep` and return a list without any empty items.
+    """
+    split = [x.strip() for x in splittable.split(sep)]
+    return [item for item in split if item]
+
+
 # Imports for backward compatibility (at bottom to avoid circular dependencies)
 from trac.core import TracError
 from trac.util.compat import reversed
