@@ -1,15 +1,15 @@
 # -*- coding: iso-8859-1 -*-
 #
-# Copyright (c) 2002, 2003, 2005 Allan Saddi <allan@saddi.com>
+# Copyright (c) 2002, 2003, 2005, 2006 Allan Saddi <allan@saddi.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Allan Saddi <allan@saddi.com>
 
@@ -19,6 +19,7 @@ fcgi - a FastCGI/WSGI gateway.
 For more information about FastCGI, see <http://www.fastcgi.com/>.
 
 For more information about the Web Server Gateway Interface, see
+
 <http://www.python.org/peps/pep-0333.html>.
 
 Example usage:
@@ -36,7 +37,7 @@ variable FCGI_FORCE_CGI to "Y" or "y".
 """
 
 __author__ = 'Allan Saddi <allan@saddi.com>'
-__version__ = '$Revision: 1797 $'
+__version__ = '$Revision: 2025 $'
 
 import sys
 import os
@@ -57,6 +58,10 @@ except ImportError:
     import dummy_threading as threading
     thread_available = False
 
+# Apparently 2.3 doesn't define SHUT_WR? Assume it is 1 in this case.
+if not hasattr(socket, 'SHUT_WR'):
+    socket.SHUT_WR = 1
+
 __all__ = ['WSGIServer']
 
 # Constants from the spec.
@@ -430,7 +435,7 @@ def encode_pair(name, value):
         s += struct.pack('!L', valueLength | 0x80000000L)
 
     return s + name + value
-    
+
 class Record(object):
     """
     A FastCGI Record.
@@ -445,6 +450,7 @@ class Record(object):
         self.paddingLength = 0
         self.contentData = ''
 
+    @staticmethod
     def _recvall(sock, length):
         """
         Attempts to receive length bytes from a socket, blocking if necessary.
@@ -468,7 +474,6 @@ class Record(object):
             recvLen += dataLen
             length -= dataLen
         return ''.join(dataList), recvLen
-    _recvall = staticmethod(_recvall)
 
     def read(self, sock):
         """Read and decode a Record from a socket."""
@@ -479,7 +484,7 @@ class Record(object):
 
         if length < FCGI_HEADER_LEN:
             raise EOFError
-        
+
         self.version, self.type, self.requestId, self.contentLength, \
                       self.paddingLength = struct.unpack(FCGI_Header, header)
 
@@ -487,7 +492,7 @@ class Record(object):
                              'contentLength = %d' %
                              (sock.fileno(), self.type, self.requestId,
                               self.contentLength))
-        
+
         if self.contentLength:
             try:
                 self.contentData, length = self._recvall(sock,
@@ -504,6 +509,7 @@ class Record(object):
             except:
                 raise EOFError
 
+    @staticmethod
     def _sendall(sock, data):
         """
         Writes data to a socket and does not return until all the data is sent.
@@ -513,16 +519,13 @@ class Record(object):
             try:
                 sent = sock.send(data)
             except socket.error, e:
-                if e[0] == errno.EPIPE:
-                    return # Don't bother raising an exception. Just ignore.
-                elif e[0] == errno.EAGAIN:
+                if e[0] == errno.EAGAIN:
                     select.select([], [sock], [])
                     continue
                 else:
                     raise
             data = data[sent:]
             length -= sent
-    _sendall = staticmethod(_sendall)
 
     def write(self, sock):
         """Encode and write a Record to a socket."""
@@ -541,7 +544,7 @@ class Record(object):
             self._sendall(sock, self.contentData)
         if self.paddingLength:
             self._sendall(sock, '\x00'*self.paddingLength)
-            
+
 class Request(object):
     """
     Represents a single FastCGI request.
@@ -576,12 +579,16 @@ class Request(object):
         if __debug__: _debug(1, 'protocolStatus = %d, appStatus = %d' %
                              (protocolStatus, appStatus))
 
-        self._flush()
-        self._end(appStatus, protocolStatus)
+        try:
+            self._flush()
+            self._end(appStatus, protocolStatus)
+        except socket.error, e:
+            if e[0] != errno.EPIPE:
+                raise
 
     def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
         self._conn.end_request(self, appStatus, protocolStatus)
-        
+
     def _flush(self):
         self.stdout.close()
         self.stderr.close()
@@ -594,14 +601,14 @@ class CGIRequest(Request):
         self.role = FCGI_RESPONDER
         self.flags = 0
         self.aborted = False
-        
+
         self.server = server
         self.params = dict(os.environ)
         self.stdin = sys.stdin
         self.stdout = StdoutWrapper(sys.stdout) # Oh, the humanity!
         self.stderr = sys.stderr
         self.data = StringIO.StringIO()
-        
+
     def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
         sys.exit(appStatus)
 
@@ -642,7 +649,7 @@ class Connection(object):
         except:
             pass
         self._sock.close()
-        
+
     def run(self):
         """Begin processing data from the socket."""
         self._keepGoing = True
@@ -738,7 +745,7 @@ class Connection(object):
                 outrec.contentData += encode_pair(name, str(cap))
 
         outrec.contentLength = len(outrec.contentData)
-        self.writeRecord(rec)
+        self.writeRecord(outrec)
 
     def _do_begin_request(self, inrec):
         """Handle an FCGI_BEGIN_REQUEST from the web server."""
@@ -802,8 +809,8 @@ class Connection(object):
         outrec = Record(FCGI_UNKNOWN_TYPE)
         outrec.contentData = struct.pack(FCGI_UnknownTypeBody, inrec.type)
         outrec.contentLength = FCGI_UnknownTypeBody_LEN
-        self.writeRecord(rec)
-        
+        self.writeRecord(outrec)
+
 class MultiplexedConnection(Connection):
     """
     A version of Connection capable of handling multiple requests
@@ -830,7 +837,7 @@ class MultiplexedConnection(Connection):
         self._lock.release()
 
         super(MultiplexedConnection, self)._cleanupSocket()
-        
+
     def writeRecord(self, rec):
         # Must use locking to prevent intermingling of Records from different
         # threads.
@@ -889,7 +896,7 @@ class MultiplexedConnection(Connection):
             super(MultiplexedConnection, self)._do_data(inrec)
         finally:
             self._lock.release()
-        
+
 class Server(object):
     """
     The FastCGI server.
@@ -909,7 +916,7 @@ class Server(object):
     inputStreamShrinkThreshold = 102400 - 8192
 
     def __init__(self, handler=None, maxwrite=8192, bindAddress=None,
-                 multiplexed=False):
+                 umask=None, multiplexed=False):
         """
         handler, if present, must reference a function or method that
         takes one argument: a Request object. If handler is not
@@ -945,7 +952,7 @@ class Server(object):
                 # Attempt to glean the maximum number of connections
                 # from the OS.
                 maxConns = resource.getrlimit(resource.RLIMIT_NOFILE)[0]
-            except ImportError:
+            except (ImportError, AttributeError):
                 maxConns = 100 # Just some made up number.
             maxReqs = maxConns
             if multiplexed:
@@ -967,6 +974,7 @@ class Server(object):
                 FCGI_MPXS_CONNS: 0
                 }
         self._bindAddress = bindAddress
+        self._umask = umask
 
     def _setupSocket(self):
         if self._bindAddress is None: # Run as a normal FastCGI?
@@ -995,6 +1003,7 @@ class Server(object):
                 sys.exit(0)
         else:
             # Run as a server
+            oldUmask = None
             if type(self._bindAddress) is str:
                 # Unix socket
                 sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
@@ -1002,6 +1011,8 @@ class Server(object):
                     os.unlink(self._bindAddress)
                 except OSError:
                     pass
+                if self._umask is not None:
+                    oldUmask = os.umask(self._umask)
             else:
                 # INET socket
                 assert type(self._bindAddress) is tuple
@@ -1012,6 +1023,9 @@ class Server(object):
             sock.bind(self._bindAddress)
             sock.listen(socket.SOMAXCONN)
 
+            if oldUmask is not None:
+                os.umask(oldUmask)
+
         return sock
 
     def _cleanupSocket(self, sock):
@@ -1028,7 +1042,7 @@ class Server(object):
     def _restoreSignalHandlers(self):
         for signum,handler in self._oldSIGs:
             signal.signal(signum, handler)
-        
+
     def _hupHandler(self, signum, frame):
         self._hupReceived = True
         self._keepGoing = False
@@ -1127,7 +1141,8 @@ class WSGIServer(Server):
     FastCGI server that supports the Web Server Gateway Interface. See
     <http://www.python.org/peps/pep-0333.html>.
     """
-    def __init__(self, application, environ=None, multithreaded=True, **kw):
+    def __init__(self, application, environ=None,
+                 multithreaded=True, **kw):
         """
         environ, if present, must be a dictionary-like object. Its
         contents will be copied into application's environ. Useful
@@ -1135,7 +1150,7 @@ class WSGIServer(Server):
 
         Set multithreaded to False if your application is not MT-safe.
         """
-        if kw.has_key('handler'):
+        if 'handler' in kw:
             del kw['handler'] # Doesn't make sense to let this through
         super(WSGIServer, self).__init__(**kw)
 
@@ -1176,7 +1191,9 @@ class WSGIServer(Server):
         environ['wsgi.multiprocess'] = True
         environ['wsgi.run_once'] = isinstance(req, CGIRequest)
 
-        if environ.get('HTTPS', 'off') in ('on', '1'):
+        if environ.get('HTTPS', '').lower() in ('yes', 'on', '1'):
+            environ['wsgi.url_scheme'] = 'https'
+        elif environ.get('HTTP_X_FORWARDED_PROTO', '').lower() == 'https':
             environ['wsgi.url_scheme'] = 'https'
         else:
             environ['wsgi.url_scheme'] = 'http'
@@ -1241,16 +1258,20 @@ class WSGIServer(Server):
         if not self.multithreaded:
             self._app_lock.acquire()
         try:
-            result = self.application(environ, start_response)
             try:
-                for data in result:
-                    if data:
-                        write(data)
-                if not headers_sent:
-                    write('') # in case body was empty
-            finally:
-                if hasattr(result, 'close'):
-                    result.close()
+                result = self.application(environ, start_response)
+                try:
+                    for data in result:
+                        if data:
+                            write(data)
+                    if not headers_sent:
+                        write('') # in case body was empty
+                finally:
+                    if hasattr(result, 'close'):
+                        result.close()
+            except socket.error, e:
+                if e[0] != errno.EPIPE:
+                    raise # Don't let EPIPE propagate beyond server
         finally:
             if not self.multithreaded:
                 self._app_lock.release()
@@ -1259,9 +1280,9 @@ class WSGIServer(Server):
 
     def _sanitizeEnv(self, environ):
         """Ensure certain values are present, if required by WSGI."""
-        if not environ.has_key('SCRIPT_NAME'):
+        if 'SCRIPT_NAME' not in environ:
             environ['SCRIPT_NAME'] = ''
-        if not environ.has_key('PATH_INFO'):
+        if 'PATH_INFO' not in environ:
             environ['PATH_INFO'] = ''
 
         # If any of these are missing, it probably signifies a broken
@@ -1270,12 +1291,12 @@ class WSGIServer(Server):
                              ('SERVER_NAME', 'localhost'),
                              ('SERVER_PORT', '80'),
                              ('SERVER_PROTOCOL', 'HTTP/1.0')]:
-            if not environ.has_key(name):
+            if name not in environ:
                 environ['wsgi.errors'].write('%s: missing FastCGI param %s '
                                              'required by WSGI!\n' %
                                              (self.__class__.__name__, name))
                 environ[name] = default
-            
+
 if __name__ == '__main__':
     def test_app(environ, start_response):
         """Probably not the most efficient example."""
