@@ -133,6 +133,7 @@ HTTP_STATUS = dict([(code, reason.title()) for code, (reason, description)
 class HTTPException(Exception):
 
     def __init__(self, detail, *args):
+        """Factory for HTTPException classes."""
         if isinstance(detail, (TracError, PermissionError)):
             self.detail = detail.message
             self.reason = detail.title
@@ -140,8 +141,9 @@ class HTTPException(Exception):
             self.detail = detail
         if args:
             self.detail = self.detail % args
-        Exception.__init__(self, '%s %s (%s)' % (self.code, self.reason,
-                                                 self.detail))
+        super(HTTPException, self).__init__('%s %s (%s)' % (self.code,
+                                                            self.reason,
+                                                            self.detail))
 
     @property
     def message(self):
@@ -356,9 +358,11 @@ class Request(object):
         raise AttributeError(name)
 
     def __repr__(self):
-        path_info = self.environ.get('PATH_INFO', '')
-        return '<%s "%s %r">' % (self.__class__.__name__, self.method,
-                                 path_info)
+        uri = self.environ.get('PATH_INFO', '')
+        qs = self.query_string
+        if qs:
+            uri += '?' + qs
+        return '<%s "%s %r">' % (self.__class__.__name__, self.method, uri)
 
     # Public API
 
@@ -490,6 +494,8 @@ class Request(object):
             self.end_headers()
             raise RequestDone
 
+    _trident_re = re.compile(r' Trident/([0-9]+)')
+
     def redirect(self, url, permanent=False):
         """Send a redirect to the client, forwarding to the specified URL.
 
@@ -514,9 +520,10 @@ class Request(object):
 
         # Workaround #10382, IE6-IE9 bug when post and redirect with hash
         if status == 303 and '#' in url:
-            match = re.search(' MSIE ([0-9]+)',
-                              self.environ.get('HTTP_USER_AGENT', ''))
-            if match and int(match.group(1)) < 10:
+            user_agent = self.environ.get('HTTP_USER_AGENT', '')
+            match_trident = self._trident_re.search(user_agent)
+            if ' MSIE ' in user_agent and \
+                    (not match_trident or int(match_trident.group(1)) < 6):
                 url = url.replace('#', '#__msie303:')
 
         self.send_header('Location', url)
@@ -533,7 +540,8 @@ class Request(object):
         self.send_header('Cache-Control', 'must-revalidate')
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.send_header('Content-Type', content_type + ';charset=utf-8')
-        self.send_header('Content-Length', len(content))
+        if isinstance(content, basestring):
+            self.send_header('Content-Length', len(content))
         self.end_headers()
 
         if self.method != 'HEAD':
@@ -619,11 +627,15 @@ class Request(object):
         self.send_header('Last-Modified', last_modified)
         use_xsendfile = getattr(self, 'use_xsendfile', False)
         if use_xsendfile:
-            self.send_header('X-Sendfile', os.path.abspath(path))
+            xsendfile_header = getattr(self, 'xsendfile_header', None)
+            if xsendfile_header:
+                self.send_header(xsendfile_header, os.path.abspath(path))
+            else:
+                use_xsendfile = False
         self.end_headers()
 
         if not use_xsendfile and self.method != 'HEAD':
-            fileobj = file(path, 'rb')
+            fileobj = open(path, 'rb')
             file_wrapper = self.environ.get('wsgi.file_wrapper', _FileWrapper)
             self._response = file_wrapper(fileobj, 4096)
         raise RequestDone
@@ -640,12 +652,15 @@ class Request(object):
         data = fileobj.read(size)
         return data
 
+    CHUNK_SIZE = 4096
+
     def write(self, data):
         """Write the given data to the response body.
 
-        *data* **must** be a `str` string, encoded with the charset
-        which has been specified in the ``'Content-Type'`` header
-        or UTF-8 otherwise.
+        *data* **must** be a `str` string or an iterable instance
+        which iterates `str` strings, encoded with the charset which
+        has been specified in the ``'Content-Type'`` header or UTF-8
+        otherwise.
 
         Note that when the ``'Content-Length'`` header is specified,
         its value either corresponds to the length of *data*, or, if
@@ -654,10 +669,26 @@ class Request(object):
         """
         if not self._write:
             self.end_headers()
-        if isinstance(data, unicode):
-            raise ValueError("Can't send unicode content")
         try:
-            self._write(data)
+            chunk_size = self.CHUNK_SIZE
+            bufsize = 0
+            buf = []
+            buf_append = buf.append
+            if isinstance(data, basestring):
+                data = [data]
+            for chunk in data:
+                if isinstance(chunk, unicode):
+                    raise ValueError("Can't send unicode content")
+                if not chunk:
+                    continue
+                bufsize += len(chunk)
+                buf_append(chunk)
+                if bufsize >= chunk_size:
+                    self._write(''.join(buf))
+                    bufsize = 0
+                    buf[:] = ()
+            if bufsize > 0:
+                self._write(''.join(buf))
         except (IOError, socket.error), e:
             if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
                 raise RequestDone
@@ -697,9 +728,14 @@ class Request(object):
 
         args = []
         for value in fs.list or ():
-            name = value.name
-            if not value.filename:
-                value = unicode(value.value, 'utf-8')
+            try:
+                name = unicode(value.name, 'utf-8')
+                if not value.filename:
+                    value = unicode(value.value, 'utf-8')
+            except UnicodeDecodeError, e:
+                raise HTTPBadRequest(
+                    _("Invalid encoding in form data: %(msg)s",
+                      msg=exception_to_unicode(e)))
             args.append((name, value))
         return args
 
