@@ -20,6 +20,7 @@
 
 from __future__ import with_statement
 
+import re
 import sys
 import time
 
@@ -137,6 +138,9 @@ class DetachedSession(dict):
             # new ones. The last concurrent request to do so "wins".
 
             if self._old != self:
+                if self._old.get('name') != self.get('name') or \
+                        self._old.get('email') != self.get('email'):
+                    self.env.invalidate_known_users_cache()
                 if not items and not authenticated:
                     # No need to keep around empty unauthenticated sessions
                     db("DELETE FROM session WHERE sid=%s AND authenticated=0",
@@ -221,9 +225,13 @@ class Session(DetachedSession):
         if sys.version_info >= (2, 6):
             self.req.outcookie[COOKIE_KEY]['httponly'] = True
 
+    _valid_sid_re = re.compile(r'[_A-Za-z0-9]+\Z')
+
     def get_session(self, sid, authenticated=False):
         refresh_cookie = False
 
+        if not authenticated and not self._valid_sid_re.match(sid):
+            raise TracError(_("Session ID must be alphanumeric."))
         if self.sid and sid != self.sid:
             refresh_cookie = True
 
@@ -241,6 +249,9 @@ class Session(DetachedSession):
         assert new_sid, 'Session ID cannot be empty'
         if new_sid == self.sid:
             return
+        if not self._valid_sid_re.match(new_sid):
+            raise TracError(_("Session ID must be alphanumeric."),
+                            _("Error renaming session"))
         with self.env.db_transaction as db:
             if db("SELECT sid FROM session WHERE sid=%s", (new_sid,)):
                 raise TracError(_("Session '%(id)s' already exists. "
@@ -438,6 +449,7 @@ class SessionAdmin(Component):
             if email is not None:
                 db("INSERT INTO session_attribute VALUES (%s,%s,'email',%s)",
                     (sid, authenticated, email))
+        self.env.invalidate_known_users_cache()
 
     def _do_set(self, attr, sid, val):
         if attr not in ('name', 'email'):
@@ -456,6 +468,7 @@ class SessionAdmin(Component):
                 """, (sid, authenticated, attr))
             db("INSERT INTO session_attribute VALUES (%s, %s, %s, %s)",
                (sid, authenticated, attr, val))
+        self.env.invalidate_known_users_cache()
 
     def _do_delete(self, *sids):
         with self.env.db_transaction as db:
@@ -473,6 +486,7 @@ class SessionAdmin(Component):
                         DELETE FROM session_attribute
                         WHERE sid=%s AND authenticated=%s
                         """, (sid, authenticated))
+        self.env.invalidate_known_users_cache()
 
     def _do_purge(self, age):
         when = parse_date(age, hint='datetime',
