@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2008 Edgewall Software
+# Copyright (C) 2005-2013 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,13 +14,7 @@
 import unittest
 
 from trac.web.tests import api, auth, cgi_frontend, chrome, href, session, \
-                           wikisyntax
-
-try:
-    import neo_cgi
-    from trac.web.tests import clearsilver
-except ImportError:
-    clearsilver = None
+                           wikisyntax, main
 
 def suite():
     suite = unittest.TestSuite()
@@ -28,11 +22,10 @@ def suite():
     suite.addTest(auth.suite())
     suite.addTest(cgi_frontend.suite())
     suite.addTest(chrome.suite())
-    if clearsilver:
-        suite.addTest(clearsilver.suite())
     suite.addTest(href.suite())
     suite.addTest(session.suite())
     suite.addTest(wikisyntax.suite())
+    suite.addTest(main.suite())
     return suite
 
 if __name__ == '__main__':
