@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2015 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -19,13 +19,15 @@ import unittest
 from StringIO import StringIO
 
 import trac.tests.compat
+from genshi.builder import tag
 from trac import perm
 from trac.core import TracError
 from trac.test import EnvironmentStub, Mock, MockPerm, locale_en
 from trac.util import create_file
 from trac.util.datefmt import utc
 from trac.util.text import shorten_line
-from trac.web.api import Request, RequestDone, parse_arg_list
+from trac.web.api import HTTPBadRequest, HTTPInternalError, Request, \
+                         RequestDone, parse_arg_list
 from tracopt.perm.authz_policy import AuthzPolicy
 
 
@@ -55,7 +57,7 @@ class RequestHandlerPermissionsTestCaseBase(unittest.TestCase):
         kw = {'perm': perm.PermissionCache(self.env, authname), 'args': {},
               'href': self.env.href, 'abs_href': self.env.abs_href,
               'tz': utc, 'locale': None, 'lc_time': locale_en,
-              'chrome': {'notices': [], 'warnings': []},
+              'authname': authname, 'chrome': {'notices': [], 'warnings': []},
               'method': None, 'get_header': lambda v: None}
         kw.update(kwargs)
         return Mock(**kw)
@@ -178,6 +180,69 @@ class RequestTestCase(unittest.TestCase):
         self.assertEqual('http://example.com/trac/test',
                          headers_sent['Location'])
 
+    def test_redirect_with_post_and_hash_for_msie(self):
+        url = 'http://example.com/trac/ticket/1#comment:2'
+        msie303 = 'http://example.com/trac/ticket/1#__msie303:comment:2'
+
+        def location(ua):
+            status_sent = []
+            headers_sent = {}
+            def start_response(status, headers):
+                status_sent.append(status)
+                headers_sent.update(dict(headers))
+            environ = self._make_environ(method='POST', HTTP_USER_AGENT=ua)
+            req = Request(environ, start_response,)
+            req.session = Mock(save=lambda: None)
+            self.assertRaises(RequestDone, req.redirect, url)
+            self.assertEqual('303 See Other', status_sent[0])
+            return headers_sent['Location']
+
+        # IE 11 strict mode
+        self.assertEqual(url, location(
+            'Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko'))
+        # IE 11 compatibility view mode
+        self.assertEqual(url, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/7.0)'))
+        # IE 10 strict mode
+        self.assertEqual(url, location(
+            'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)'
+            ))
+        # IE 10 compatibility view mode
+        self.assertEqual(url, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0)'))
+        # IE 9 strict mode
+        self.assertEqual(msie303, location(
+            'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)'))
+        # IE 9 compatibility view mode
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0)'))
+        # IE 8 strict mode
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)'))
+        # IE 8 compatibility view mode
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)'))
+        # IE 7
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)'))
+        # IE 6
+        self.assertEqual(msie303, location(
+            'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)'))
+
+    def test_write_iterable(self):
+        buf = StringIO()
+        def write(data):
+            buf.write(data)
+        def start_response(status, headers):
+            return write
+        environ = self._make_environ(method='GET')
+
+        buf = StringIO()
+        req = Request(environ, start_response)
+        req.send_header('Content-Type', 'text/plain;charset=utf-8')
+        req.write(('Foo', 'bar', 'baz'))
+        self.assertEqual('Foobarbaz', buf.getvalue())
+
     def test_write_unicode(self):
         buf = StringIO()
         def write(data):
@@ -191,6 +256,32 @@ class RequestTestCase(unittest.TestCase):
         req.send_header('Content-Length', 0)
         # anyway we're not supposed to send unicode, so we get a ValueError
         self.assertRaises(ValueError, req.write, u'Föö')
+        self.assertRaises(ValueError, req.write, ('F', u'öo'))
+
+    def test_send_iterable(self):
+        baton = {'content': StringIO(), 'status': None, 'headers': None}
+        def write(data):
+            baton['content'].write(data)
+        def start_response(status, headers):
+            baton['status'] = status
+            baton['headers'] = headers
+            return write
+        environ = self._make_environ(method='GET')
+
+        def iterable():
+            yield 'line1,'
+            yield ''
+            yield 'line2,'
+            yield 'line3\n'
+
+        req = Request(environ, start_response)
+        self.assertRaises(RequestDone, req.send, iterable())
+        self.assertEqual('200 Ok', baton['status'])
+        self.assertEqual([('Cache-Control', 'must-revalidate'),
+                          ('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT'),
+                          ('Content-Type', 'text/html;charset=utf-8')],
+                         baton['headers'])
+        self.assertEqual('line1,line2,line3\n', baton['content'].getvalue())
 
     def test_invalid_cookies(self):
         environ = self._make_environ(HTTP_COOKIE='bad:key=value;')
@@ -229,6 +320,93 @@ class RequestTestCase(unittest.TestCase):
         req = Request(environ, None)
         self.assertEqual('bar', req.args['action'])
 
+    def test_qs_invalid_value_bytes(self):
+        environ = self._make_environ(**{'QUERY_STRING': 'name=%FF'})
+        req = Request(environ, None)
+        self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
+
+    def test_qs_invalid_name_bytes(self):
+        environ = self._make_environ(**{'QUERY_STRING': '%FF=value'})
+        req = Request(environ, None)
+        self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
+
+
+class RequestSendFileTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.status = None
+        self.headers = None
+        self.response = StringIO()
+        self.dir = tempfile.mkdtemp(prefix='trac-')
+        self.filename = os.path.join(self.dir, 'test.txt')
+        self.data = 'contents\n'
+        create_file(self.filename, self.data, 'wb')
+        self.req = None
+
+    def tearDown(self):
+        if self.req and self.req._response:
+            self.req._response.close()
+        shutil.rmtree(self.dir)
+
+    def _start_response(self, status, headers):
+        self.status = status
+        self.headers = dict(headers)
+        def write(data):
+            self.response.write(data)
+        return write
+
+    def _create_req(self, use_xsendfile=False, xsendfile_header='X-Sendfile',
+                    **kwargs):
+        req = Request(_make_environ(**kwargs), self._start_response)
+        req.callbacks.update({'use_xsendfile': lambda r: use_xsendfile,
+                              'xsendfile_header': lambda r: xsendfile_header})
+        self.req = req
+        return req
+
+    def test_send_file(self):
+        req = self._create_req()
+        self.assertRaises(RequestDone, req.send_file, self.filename,
+                          'text/plain')
+        self.assertEqual('200 Ok', self.status)
+        self.assertEqual('text/plain', self.headers['Content-Type'])
+        self.assertEqual(str(len(self.data)), self.headers['Content-Length'])
+        self.assertNotIn('X-Sendfile', self.headers)
+        self.assertEqual(self.data, ''.join(req._response))
+        self.assertEqual('', self.response.getvalue())
+
+    def test_send_file_with_xsendfile(self):
+        req = self._create_req(use_xsendfile=True)
+        self.assertRaises(RequestDone, req.send_file, self.filename,
+                          'text/plain')
+        self.assertEqual('200 Ok', self.status)
+        self.assertEqual('text/plain', self.headers['Content-Type'])
+        self.assertEqual(self.filename, self.headers['X-Sendfile'])
+        self.assertEqual(None, req._response)
+        self.assertEqual('', self.response.getvalue())
+
+    def test_send_file_with_xsendfile_header(self):
+        req = self._create_req(use_xsendfile=True,
+                               xsendfile_header='X-Accel-Redirect')
+        self.assertRaises(RequestDone, req.send_file, self.filename,
+                          'text/plain')
+        self.assertEqual('200 Ok', self.status)
+        self.assertEqual('text/plain', self.headers['Content-Type'])
+        self.assertEqual(self.filename, self.headers['X-Accel-Redirect'])
+        self.assertNotIn('X-Sendfile', self.headers)
+        self.assertEqual(None, req._response)
+        self.assertEqual('', self.response.getvalue())
+
+    def test_send_file_with_xsendfile_and_empty_header(self):
+        req = self._create_req(use_xsendfile=True, xsendfile_header='')
+        self.assertRaises(RequestDone, req.send_file, self.filename,
+                          'text/plain')
+        self.assertEqual('200 Ok', self.status)
+        self.assertEqual('text/plain', self.headers['Content-Type'])
+        self.assertEqual(str(len(self.data)), self.headers['Content-Length'])
+        self.assertNotIn('X-Sendfile', self.headers)
+        self.assertEqual(self.data, ''.join(req._response))
+        self.assertEqual('', self.response.getvalue())
+
 
 class SendErrorTestCase(unittest.TestCase):
 
@@ -363,11 +541,38 @@ class ParseArgListTestCase(unittest.TestCase):
         self.assertEqual(u'résu&mé', args[1][0])
 
 
+class HTTPExceptionTestCase(unittest.TestCase):
+
+    def test_tracerror_with_string_as_argument(self):
+        e1 = TracError('the message')
+        e2 = HTTPInternalError(e1)
+        self.assertEqual('500 Trac Error (the message)', unicode(e2))
+
+    def test_tracerror_with_fragment_as_argument(self):
+        e1 = TracError(tag(tag.b('the message')))
+        e2 = HTTPInternalError(e1)
+        self.assertEqual('500 Trac Error (<b>the message</b>)', unicode(e2))
+
+    def test_exception_with_string_as_argument(self):
+        e1 = Exception('the message')
+        e2 = HTTPInternalError(e1)
+        self.assertEqual('500 Internal Server Error (the message)',
+                         unicode(e2))
+
+    def test_exception_with_fragment_as_argument(self):
+        e1 = Exception(tag(tag.b('the message')))
+        e2 = HTTPInternalError(e1)
+        self.assertEqual('500 Internal Server Error (<b>the message</b>)',
+                         unicode(e2))
+
+
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(RequestTestCase))
+    suite.addTest(unittest.makeSuite(RequestSendFileTestCase))
     suite.addTest(unittest.makeSuite(SendErrorTestCase))
     suite.addTest(unittest.makeSuite(ParseArgListTestCase))
+    suite.addTest(unittest.makeSuite(HTTPExceptionTestCase))
     return suite
 
 
