@@ -42,8 +42,8 @@ class TestWikiChangeListener(Component):
     def wiki_page_added(self, page):
         self.added.append(page)
 
-    def wiki_page_changed(self, page, version, t, comment, author, ipnr):
-        self.changed.append((page, version, t, comment, author, ipnr))
+    def wiki_page_changed(self, page, version, t, comment, author):
+        self.changed.append((page, version, t, comment, author))
 
     def wiki_page_deleted(self, page):
         self.deleted.append(page)
@@ -55,6 +55,12 @@ class TestWikiChangeListener(Component):
         self.renamed.append((page, old_name))
 
 
+class TestLegacyWikiChangeListener(TestWikiChangeListener):
+
+    def wiki_page_changed(self, page, version, t, comment, author, ipnr):
+        self.changed.append((page, version, t, comment, author, ipnr))
+
+
 class WikiPageTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -153,7 +159,7 @@ class WikiPageTestCase(unittest.TestCase):
         with self.env.db_query as db:
             rows = db("""
                SELECT version, time, author, ipnr, text, comment, readonly
-               FROM wiki WHERE name=%s
+               FROM wiki WHERE name=%s ORDER BY version
                """, ('TestPage',))
             self.assertEqual(2, len(rows))
             self.assertEqual((1, to_utimestamp(t), 'joe', '::1', 'Bla bla',
@@ -161,9 +167,12 @@ class WikiPageTestCase(unittest.TestCase):
             self.assertEqual((2, to_utimestamp(t2), 'kate', '192.168.0.101',
                               'Bla', 'Changing', 0), rows[1])
 
-        listener = TestWikiChangeListener(self.env)
+        listener = TestLegacyWikiChangeListener(self.env)
         self.assertEqual((page, 2, t2, 'Changing', 'kate', '192.168.0.101'),
                          listener.changed[0])
+        listener = TestWikiChangeListener(self.env)
+        self.assertEqual((page, 2, t2, 'Changing', 'kate'),
+                         listener.changed[0])
 
         page = WikiPage(self.env, 'TestPage')
         history = list(page.get_history())
