@@ -59,7 +59,7 @@ from trac.config import ListOption, ChoiceOption
 from trac.core import *
 from trac.env import ISystemInfoProvider
 from trac.versioncontrol import Changeset, Node, Repository, \
-                                IRepositoryConnector, \
+                                IRepositoryConnector, InvalidRepository, \
                                 NoSuchChangeset, NoSuchNode
 from trac.versioncontrol.cache import CachedRepository
 from trac.util import embedded_numbers
@@ -67,6 +67,7 @@ from trac.util.concurrency import threading
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _
 from trac.util.datefmt import from_utimestamp, to_datetime, utc
+from trac.web.href import Href
 
 
 application_pool = None
@@ -294,7 +295,7 @@ class SubversionConnector(Component):
         self._version = None
         try:
             _import_svn()
-            self.log.debug('Subversion bindings imported')
+            self.log.debug("Subversion bindings imported")
         except ImportError, e:
             self.error = e
             self.log.info('Failed to load Subversion bindings', exc_info=True)
@@ -358,15 +359,17 @@ class SubversionRepository(Repository):
 
         root_path_utf8 = repos.svn_repos_find_root_path(path_utf8, self.pool())
         if root_path_utf8 is None:
-            raise TracError(_("%(path)s does not appear to be a Subversion "
-                              "repository.", path=to_unicode(path_utf8)))
+            raise InvalidRepository(
+                _("%(path)s does not appear to be a Subversion repository.",
+                  path=to_unicode(path_utf8)))
 
         try:
             self.repos = repos.svn_repos_open(root_path_utf8, self.pool())
         except core.SubversionException, e:
-            raise TracError(_("Couldn't open Subversion repository %(path)s: "
-                              "%(svn_error)s", path=to_unicode(path_utf8),
-                              svn_error=exception_to_unicode(e)))
+            raise InvalidRepository(
+                _("Couldn't open Subversion repository %(path)s: "
+                  "%(svn_error)s", path=to_unicode(path_utf8),
+                  svn_error=exception_to_unicode(e)))
         self.fs_ptr = repos.svn_repos_fs(self.repos)
 
         self.uuid = fs.get_uuid(self.fs_ptr, self.pool())
@@ -482,9 +485,10 @@ class SubversionRepository(Repository):
         """
         url = self.params.get('url', '').rstrip('/')
         if url:
-            if not path or path == '/':
-                return url
-            return url + '/' + path.lstrip('/')
+            href = Href(url)
+            if path:
+                path = path.lstrip('/')
+            return href(path)
 
     def get_changeset(self, rev):
         """Produce a `SubversionChangeset` from given revision
@@ -1145,9 +1149,11 @@ class FileContentStream(object):
         'header': ['Header'],
         }
     KEYWORDS = reduce(set.union, map(set, KEYWORD_GROUPS.values()))
+    KEYWORD_SPLIT_RE = re.compile(r'[ \t\v\n\b\r\f]+')
+    KEYWORD_EXPAND_RE = re.compile(r'%[abdDPrRu_%HI]')
     NATIVE_EOL = '\r\n' if os.name == 'nt' else '\n'
     NEWLINES = {'LF': '\n', 'CRLF': '\r\n', 'CR': '\r', 'native': NATIVE_EOL}
-    KEYWORD_MAX_SIZE = 256
+    KEYWORD_MAX_SIZE = 255
     CHUNK_SIZE = 4096
 
     keywords_re = None
@@ -1164,9 +1170,8 @@ class FileContentStream(object):
         # Note: we _must_ use a detached pool here, as the lifetime of
         # this object can exceed those of the node or even the repository
         if keyword_substitution:
-            keywords = (node._get_prop(core.SVN_PROP_KEYWORDS) or '').split()
-            self.keywords = self._get_keyword_values(set(keywords) &
-                                                 set(self.KEYWORDS))
+            self.keywords = self._get_keyword_values(
+                                        node._get_prop(core.SVN_PROP_KEYWORDS))
             self.keywords_re = self._build_keywords_re(self.keywords)
         if self.NEWLINES.get(eol, '\n') != '\n' and \
            node._get_prop(core.SVN_PROP_EOL_STYLE) == 'native':
@@ -1197,30 +1202,51 @@ class FileContentStream(object):
     def _get_revprop(self, name):
         return fs.revision_prop(self.fs_ptr, self.node.rev, name, self.pool())
 
+    def _split_keywords(self, keywords):
+        return filter(None, self.KEYWORD_SPLIT_RE.split(keywords or ''))
+
     def _get_keyword_values(self, keywords):
+        keywords = self._split_keywords(keywords)
         if not keywords:
             return None
 
         node = self.node
         mtime = to_datetime(node.last_modified, utc)
         shortdate = mtime.strftime('%Y-%m-%d %H:%M:%SZ')
+        longdate = mtime.strftime('%Y-%m-%d %H:%M:%S +0000 (%a, %d %b %Y)')
         created_rev = unicode(node.created_rev)
         # Note that the `to_unicode` has a small probability to mess-up binary
         # properties, see #4321.
         author = to_unicode(self._get_revprop(core.SVN_PROP_REVISION_AUTHOR))
-        url = node.repos.get_path_url(node.path, node.rev) or node.path
+        path = node.path.lstrip('/')
+        url = node.repos.get_path_url(path, node.rev) or path
+        root_url = node.repos.get_path_url('', node.rev) or '/'
+        id_ = ' '.join((node.name, created_rev, shortdate, author))
         data = {
-            'rev': created_rev, 'author': author, 'url': url,
-            'date': mtime.strftime('%Y-%m-%d %H:%M:%S +0000 (%a, %d %b %Y)'),
-            'id': ' '.join((posixpath.basename(node.path), created_rev,
-                            shortdate, author)),
+            'rev': created_rev, 'author': author, 'url': url, 'date': longdate,
+            'id': id_,
             'header': ' '.join((url, created_rev, shortdate, author)),
-            }
+            '%a': author, '%b': node.name, '%d': shortdate, '%D': longdate,
+            '%P': path, '%r': created_rev, '%R': root_url, '%u': url,
+            '%_': ' ', '%%': '%', '%I': id_,
+            '%H': ' '.join((path, created_rev, shortdate, author)),
+        }
+
+        def expand(match):
+            match = match.group(0)
+            return data.get(match, match)
+
         values = {}
         for name, aliases in self.KEYWORD_GROUPS.iteritems():
-            if any(kw for kw in aliases if kw in keywords):
-                for kw in aliases:
-                    values[kw] = data[name]
+            if any(kw in keywords for kw in aliases):
+                values.update((kw, data[name]) for kw in aliases)
+        for keyword in keywords:
+            if '=' not in keyword:
+                continue
+            name, definition = keyword.split('=', 1)
+            if name not in self.KEYWORDS:
+                values[name] = self.KEYWORD_EXPAND_RE.sub(expand, definition)
+
         if values:
             return dict((key, value.encode('utf-8'))
                         for key, value in values.iteritems())
@@ -1232,12 +1258,11 @@ class FileContentStream(object):
             return re.compile("""
                 [$]
                 (?P<keyword>%s)
-                (?P<rest>
-                    (?: :[ ][^$\r\n]+?[ ]
-                    |   ::[ ][^$\r\n]+?[ #]
-                    )
+                (?:
+                    :[ ][^$\r\n]+?[ ]   |
+                    ::[ ](?P<fixed>[^$\r\n]+?)[ #]
                 )?
-                [$]""" % '|'.join(keywords),
+                [$]""" % '|'.join(map(re.escape, keywords)),
                 re.VERBOSE)
         else:
             return None
@@ -1304,15 +1329,16 @@ class FileContentStream(object):
             data = data.replace('\n', self.newline)
         return data
 
-    def _translate_keyword(self, buffer, match):
+    def _translate_keyword(self, text, match):
         keyword = match.group('keyword')
         value = self.keywords.get(keyword)
         if value is None:
-            return buffer
-        rest = match.group('rest')
-        if rest is None or not rest.startswith('::'):
-            return '$%s: %s $' % (keyword, value)
-        elif len(rest) - 4 >= len(value):
-            return '$%s:: %-*s $' % (keyword, len(rest) - 4, value)
+            return text
+        fixed = match.group('fixed')
+        if fixed is None:
+            n = self.KEYWORD_MAX_SIZE - len(keyword) - 5
+            return '$%s: %.*s $' % (keyword, n, value) if n >= 0 else text
         else:
-            return '$%s:: %s#$' % (keyword, value[:len(rest) - 4])
+            n = len(fixed)
+            return '$%s:: %-*.*s%s$' % \
+                   (keyword, n, n, value, '#' if n < len(value) else ' ')
