@@ -27,6 +27,7 @@ from trac.versioncontrol.web_ui.browser import IPropertyRenderer
 from trac.versioncontrol.web_ui.changeset import IPropertyDiffRenderer
 from trac.util import Ranges, to_ranges
 from trac.util.translation import _, tag_
+from trac.web.chrome import chrome_resource_path
 from tracopt.versioncontrol.svn.svn_fs import _path_within_scope
 
 
@@ -90,14 +91,17 @@ class SubversionPropertyRenderer(Component):
         elif name == 'svn:mergeinfo' or name.startswith('svnmerge-'):
             return self._render_mergeinfo(name, mode, context, props)
 
+    def _is_abs_url(self, url):
+        return url and '://' in url
+
     def _render_externals(self, prop):
         if not self._externals_map:
             for dummykey, value in self.svn_externals_section.options():
                 value = value.split()
                 if len(value) != 2:
                     self.log.warn("svn:externals entry %s doesn't contain "
-                            "a space-separated key value pair, skipping.",
-                            dummykey)
+                                  "a space-separated key value pair, "
+                                  "skipping.", dummykey)
                     continue
                 key, value = value
                 self._externals_map[key] = value.replace('%', '%%') \
@@ -116,6 +120,9 @@ class SubversionPropertyRenderer(Component):
                 rev = elements[1]
                 rev = rev.replace('-r', '')
             # retrieve a matching entry in the externals map
+            if not self._is_abs_url(url):
+                externals.append((external, None, None, None, None))
+                continue
             prefix = []
             base_url = url
             while base_url:
@@ -153,8 +160,8 @@ class SubversionPropertyRenderer(Component):
                        for label, href, title in externals_data])
 
     def _render_needslock(self, context):
-        return tag.img(src=context.href.chrome('common/lock-locked.png'),
-                       alt=_("needs lock"), title=_("needs lock"))
+        url = chrome_resource_path(context.req, 'common/lock-locked.png')
+        return tag.img(src=url, alt=_("needs lock"), title=_("needs lock"))
 
     def _render_mergeinfo(self, name, mode, context, props):
         rows = []
@@ -266,10 +273,12 @@ class SubversionMergePropertyRenderer(Component):
         if not rows:
             return None
         rows.sort()
-        has_deleted = rows[-1][0] if rows else None
-        return tag(has_deleted and tag.a(_('(toggle deleted branches)'),
-                                         class_='trac-toggledeleted',
-                                         href='#'),
+        if rows and rows[-1][0]:
+            toggledeleted = tag.a(_("(toggle deleted branches)"),
+                                  class_='trac-toggledeleted', href='#')
+        else:
+            toggledeleted = None
+        return tag(toggledeleted,
                    tag.table(tag.tbody(
                        [tag.tr(row, class_='trac-deleted' if deleted else None)
                         for deleted, spath, row in rows]), class_='props'))
@@ -402,9 +411,9 @@ class SubversionMergePropertyDiffRenderer(Component):
                 revs = set(changed_revs[spath])
                 added &= revs
                 removed &= revs
-            if added or removed:
                 added_ni &= revs
                 removed_ni &= revs
+            if added or removed:
                 if new_spath:
                     status = _(" (added)")
                 else:
